@@ -19,7 +19,25 @@
 
 package confdb
 
-var GetValuesThroughPaths = getValuesThroughPaths
+type (
+	ViewRef = viewRef
+)
+
+var (
+	GetValuesThroughPaths = getValuesThroughPaths
+	NewAuthentication     = newAuthentication
+)
+
+type Authentication = authentication
+
+func (a Authentication) ToStrings() []string {
+	return a.toStrings()
+}
+
+func ParsePathIntoAccessors(path string) ([]accessor, error) {
+	opts := parseOpts{pathType: viewPath}
+	return parsePathIntoAccessors(path, opts)
+}
 
 func MockMaxValueDepth(newDepth int) (restore func()) {
 	oldDepth := maxValueDepth
@@ -28,9 +46,3 @@ func MockMaxValueDepth(newDepth int) (restore func()) {
 		maxValueDepth = oldDepth
 	}
 }
-
-// isValidAuthenticationMethod exposed for tests
-var IsValidAuthenticationMethod = isValidAuthenticationMethod
-
-// convertToAuthenticationMethods exposed for tests
-var ConvertToAuthenticationMethods = convertToAuthenticationMethods
