@@ -23,7 +23,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"sort"
@@ -33,6 +32,7 @@ import (
 
 	"github.com/snapcore/snapd/arch"
 	"github.com/snapcore/snapd/asserts"
+	"github.com/snapcore/snapd/asserts/snapasserts"
 	"github.com/snapcore/snapd/asserts/sysdb"
 	"github.com/snapcore/snapd/boot"
 	"github.com/snapcore/snapd/dirs"
@@ -47,6 +47,7 @@ import (
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/seed/seedwriter"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/snap/snapfile"
 	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/sysconfig"
@@ -152,6 +153,23 @@ func Prepare(opts *Options) error {
 		return err
 	}
 
+	for _, assertionsFilename := range opts.ExtraAssertionsFiles {
+		// Function reads the assertions from the file, decodes them and rejects
+		// assertion types that are not allowed
+
+		assertionsFile, err := os.Open(assertionsFilename)
+		if err != nil {
+			return fmt.Errorf("cannot read extra assertion: %s", err)
+		}
+		defer assertionsFile.Close()
+		extraAssertions, err := decodeExtraAssertions(assertionsFile)
+		if err != nil {
+			return err
+		}
+
+		opts.ExtraAssertions = append(opts.ExtraAssertions, extraAssertions...)
+	}
+
 	if err := setupSeed(tsto, model, opts); err != nil {
 		return err
 	}
@@ -161,8 +179,13 @@ func Prepare(opts *Options) error {
 		if model.Classic() {
 			return fmt.Errorf("cannot preseed the image for a classic model")
 		}
-		if model.Base() != "core20" {
-			return fmt.Errorf("cannot preseed the image for a model other than core20")
+
+		coreVersion, err := naming.CoreVersion(model.Base())
+		if err != nil {
+			return fmt.Errorf("cannot preseed the image for %s: %v", model.Base(), err)
+		}
+		if coreVersion < 20 {
+			return fmt.Errorf("cannot preseed the image for older base than core20")
 		}
 		coreOpts := &preseed.CoreOptions{
 			PrepareImageDir:           opts.PrepareDir,
@@ -183,7 +206,7 @@ var reserved = []string{"core", "os", "class", "allowed-modes"}
 func decodeModelAssertion(opts *Options) (*asserts.Model, error) {
 	fn := opts.ModelFile
 
-	rawAssert, err := ioutil.ReadFile(fn)
+	rawAssert, err := os.ReadFile(fn)
 	if err != nil {
 		return nil, fmt.Errorf("cannot read model assertion: %s", err)
 	}
@@ -237,7 +260,7 @@ func customizeImage(rootDir, defaultsDir string, custo *Customizations) error {
 		if err := os.MkdirAll(varCloudDir, 0755); err != nil {
 			return err
 		}
-		if err := ioutil.WriteFile(filepath.Join(varCloudDir, "meta-data"), []byte("instance-id: nocloud-static\n"), 0644); err != nil {
+		if err := os.WriteFile(filepath.Join(varCloudDir, "meta-data"), []byte("instance-id: nocloud-static\n"), 0644); err != nil {
 			return err
 		}
 		dst := filepath.Join(varCloudDir, "user-data")
@@ -252,7 +275,7 @@ func customizeImage(rootDir, defaultsDir string, custo *Customizations) error {
 		if err := os.MkdirAll(filepath.Dir(consoleConfDisabled), 0755); err != nil {
 			return err
 		}
-		if err := ioutil.WriteFile(consoleConfDisabled, []byte("console-conf has been disabled by image customization\n"), 0644); err != nil {
+		if err := os.WriteFile(consoleConfDisabled, []byte("console-conf has been disabled by image customization\n"), 0644); err != nil {
 			return err
 		}
 	}
@@ -278,19 +301,21 @@ type imageSeeder struct {
 	model *asserts.Model
 	tsto  *tooling.ToolingStore
 
-	classic        bool
-	prepareDir     string
-	wideCohortKey  string
-	revisions      map[string]snap.Revision
-	customizations *Customizations
-	architecture   string
+	classic                  bool
+	prepareDir               string
+	wideCohortKey            string
+	customizations           *Customizations
+	architecture             string
+	allowSnapdKernelMismatch bool
 
 	hasModes    bool
 	rootDir     string
 	bootRootDir string
 	seedDir     string
 	label       string
+	db          *asserts.Database
 	w           *seedwriter.Writer
+	f           seedwriter.SeedAssertionFetcher
 }
 
 func newImageSeeder(tsto *tooling.ToolingStore, model *asserts.Model, opts *Options) (*imageSeeder, error) {
@@ -304,17 +329,21 @@ func newImageSeeder(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opti
 		classic:       opts.Classic,
 		prepareDir:    opts.PrepareDir,
 		wideCohortKey: opts.WideCohortKey,
-		revisions:     opts.Revisions,
 		// keep a pointer to the customization object in opts as the Validation
 		// member might be defaulted if not set.
-		customizations: &opts.Customizations,
-		architecture:   determineImageArchitecture(model, opts),
+		customizations:           &opts.Customizations,
+		architecture:             determineImageArchitecture(model, opts),
+		allowSnapdKernelMismatch: opts.AllowSnapdKernelMismatch,
 
 		hasModes: model.Grade() != asserts.ModelGradeUnset,
 		model:    model,
 		tsto:     tsto,
 	}
 
+	if os.Getenv("SNAPD_ALLOW_SNAPD_KERNEL_MISMATCH") == "true" {
+		s.allowSnapdKernelMismatch = true
+	}
+
 	if !s.hasModes {
 		if err := s.setModelessDirs(); err != nil {
 			return nil, err
@@ -340,8 +369,12 @@ func newImageSeeder(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opti
 		SeedDir:        s.seedDir,
 		Label:          s.label,
 		DefaultChannel: opts.Channel,
+		Manifest:       opts.SeedManifest,
+		ManifestPath:   opts.SeedManifestPath,
 
 		TestSkipCopyUnverifiedModel: osutil.GetenvBool("UBUNTU_IMAGE_SKIP_COPY_UNVERIFIED_MODEL"),
+
+		ExtraAssertions: opts.ExtraAssertions,
 	}
 	w, err := seedwriter.New(model, wOpts)
 	if err != nil {
@@ -399,14 +432,27 @@ func (s *imageSeeder) setModesDirs() error {
 	return nil
 }
 
-func (s *imageSeeder) start(db *asserts.Database, optSnaps []*seedwriter.OptionsSnap) (seedwriter.RefAssertsFetcher, error) {
+func (s *imageSeeder) start(optSnaps []*seedwriter.OptionsSnap) error {
 	if err := s.w.SetOptionsSnaps(optSnaps); err != nil {
-		return nil, err
+		return err
 	}
+
+	// TODO: developer database in home or use snapd (but need
+	// a bit more API there, potential issues when crossing stores/series)
+	db, err := asserts.OpenDatabase(&asserts.DatabaseConfig{
+		Backstore: asserts.NewMemoryBackstore(),
+		Trusted:   trusted,
+	})
+	if err != nil {
+		return err
+	}
+
 	newFetcher := func(save func(asserts.Assertion) error) asserts.Fetcher {
-		return s.tsto.AssertionFetcher(db, save)
+		return s.tsto.AssertionSequenceFormingFetcher(db, save)
 	}
-	return s.w.Start(db, newFetcher)
+	s.db = db
+	s.f = seedwriter.MakeSeedAssertionFetcher(newFetcher)
+	return s.w.Start(db, s.f)
 }
 
 func (s *imageSeeder) snapSupportsImageArch(sn *seedwriter.SeedSnap) bool {
@@ -430,18 +476,31 @@ func (s *imageSeeder) validateSnapArchs(snaps []*seedwriter.SeedSnap) error {
 
 type localSnapRefs map[*seedwriter.SeedSnap][]*asserts.Ref
 
-func (s *imageSeeder) deriveInfoForLocalSnaps(f seedwriter.RefAssertsFetcher, db *asserts.Database) (localSnapRefs, error) {
+func (s *imageSeeder) deriveInfoForLocalSnaps(localCompsPaths []string, f seedwriter.SeedAssertionFetcher, db *asserts.Database) (localSnapRefs, error) {
 	localSnaps, err := s.w.LocalSnaps()
 	if err != nil {
 		return nil, err
 	}
 
-	snaps := make(map[*seedwriter.SeedSnap][]*asserts.Ref)
-	for _, sn := range localSnaps {
-		si, aRefs, err := seedwriter.DeriveSideInfo(sn.Path, s.model, f, db)
-		if err != nil && !errors.Is(err, &asserts.NotFoundError{}) {
+	cinfos := make(map[string]*snap.ComponentInfo, len(localCompsPaths))
+	for _, path := range localCompsPaths {
+		ci, err := readComponentInfoFromCont(path)
+		if err != nil {
 			return nil, err
 		}
+		cinfos[path] = ci
+	}
+
+	snaps := make(localSnapRefs)
+	for _, sn := range localSnaps {
+		assertedSnap := true
+		si, aRefs, err := seedwriter.DeriveSideInfo(sn.Path, s.model, f, db)
+		if err != nil {
+			if !errors.Is(err, &asserts.NotFoundError{}) {
+				return nil, err
+			}
+			assertedSnap = false
+		}
 
 		snapFile, err := snapfile.Open(sn.Path)
 		if err != nil {
@@ -452,12 +511,53 @@ func (s *imageSeeder) deriveInfoForLocalSnaps(f seedwriter.RefAssertsFetcher, db
 			return nil, err
 		}
 
-		if err := s.w.SetInfo(sn, info); err != nil {
+		// Assign components now that we know the snap name
+		seedComps := map[string]*seedwriter.SeedComponent{}
+		for path, ci := range cinfos {
+			if ci.Component.SnapName != info.SnapName() {
+				continue
+			}
+
+			if assertedSnap {
+				// Components for an asserted snap should have
+				// assertions too, error out otherwise
+				csi, crefs, err := seedwriter.DeriveComponentSideInfo(
+					path, ci, info, s.model, f, db)
+				if err != nil {
+					return nil, err
+				}
+				ci.ComponentSideInfo = *csi
+				aRefs = append(aRefs, crefs...)
+			}
+			seedComps[ci.Component.ComponentName] = &seedwriter.SeedComponent{
+				ComponentRef: naming.NewComponentRef(info.SnapName(),
+					ci.Component.ComponentName),
+				Path: path,
+				Info: ci,
+			}
+			delete(cinfos, path)
+		}
+
+		// For local snaps, the component information is set inside
+		// w.SetInfo by looking at the local components information set
+		// in the call to w.SetOptionsSnaps.
+		if err := s.w.SetInfo(sn, info, seedComps); err != nil {
 			return nil, err
 		}
+
 		snaps[sn] = aRefs
 	}
 
+	// Check if there are local components that did not belong to one
+	// of the local snaps
+	errMsg := ""
+	for path := range cinfos {
+		errMsg += fmt.Sprintf("\n%q local component does not have a matching local snap", path)
+	}
+	if errMsg != "" {
+		return nil, fmt.Errorf("missing local snaps:%s", errMsg)
+	}
+
 	// derive info first before verifying the arch
 	if err := s.validateSnapArchs(localSnaps); err != nil {
 		return nil, err
@@ -465,34 +565,114 @@ func (s *imageSeeder) deriveInfoForLocalSnaps(f seedwriter.RefAssertsFetcher, db
 	return snaps, s.w.InfoDerived()
 }
 
+func (s *imageSeeder) validationSetKeysAndRevisionForSnap(snapName string) ([]snapasserts.ValidationSetKey, snap.Revision, error) {
+	vsas, err := s.db.FindMany(asserts.ValidationSetType, nil)
+	if err != nil && !errors.Is(err, &asserts.NotFoundError{}) {
+		return nil, snap.Revision{}, err
+	}
+
+	allVss := snapasserts.NewValidationSets()
+	for _, a := range vsas {
+		if err := allVss.Add(a.(*asserts.ValidationSet)); err != nil {
+			return nil, snap.Revision{}, err
+		}
+	}
+
+	// Just for a good measure, perform a conflict check once we have the
+	// list of all validation-sets for the image seed.
+	if err := allVss.Conflict(); err != nil {
+		return nil, snap.Revision{}, err
+	}
+
+	pres, err := allVss.Presence(naming.Snap(snapName))
+	if err != nil {
+		return nil, snap.Revision{}, err
+	}
+
+	// TODO: figure out if this is needed
+	if pres.Presence == asserts.PresenceInvalid {
+		return nil, snap.Revision{}, fmt.Errorf("snap %q is invalid in validation sets: %v", snapName, pres.Sets.CommaSeparated())
+	}
+
+	if pres.Constrained() {
+		return pres.Sets, pres.Revision, nil
+	}
+	return nil, s.w.Manifest().AllowedSnapRevision(snapName), nil
+}
+
 func (s *imageSeeder) downloadSnaps(snapsToDownload []*seedwriter.SeedSnap, curSnaps []*tooling.CurrentSnap) (downloadedSnaps map[string]*tooling.DownloadedSnap, err error) {
 	byName := make(map[string]*seedwriter.SeedSnap, len(snapsToDownload))
-	beforeDownload := func(info *snap.Info) (string, error) {
+	revisions := make(map[string]snap.Revision)
+	beforeDownload := func(info *snap.Info, cinfos map[string]*snap.ComponentInfo) (string, map[string]string, error) {
 		sn := byName[info.SnapName()]
 		if sn == nil {
-			return "", fmt.Errorf("internal error: downloading unexpected snap %q", info.SnapName())
+			return "", nil, fmt.Errorf("internal error: downloading unexpected snap %q", info.SnapName())
 		}
-		rev := s.revisions[sn.SnapName()]
-		if !rev.Unset() {
-			fmt.Fprintf(Stdout, "Fetching %s (%d)\n", sn.SnapName(), rev)
-		} else {
-			fmt.Fprintf(Stdout, "Fetching %s (%d)\n", sn.SnapName(), info.Revision)
+		rev := revisions[info.SnapName()]
+		if rev.Unset() {
+			rev = info.Revision
 		}
-		if err := s.w.SetInfo(sn, info); err != nil {
-			return "", err
+		seedComps := make(map[string]*seedwriter.SeedComponent, len(cinfos))
+		for _, ci := range cinfos {
+			// No path as these are downloaded components
+			seedComps[ci.Component.ComponentName] = &seedwriter.SeedComponent{
+				ComponentRef: ci.Component,
+				Path:         "",
+				Info:         ci,
+			}
+		}
+		fmt.Fprintf(Stdout, "Fetching %s (%s)\n", sn.SnapName(), rev)
+		if err := s.w.SetInfo(sn, info, seedComps); err != nil {
+			return "", nil, err
 		}
 		if err := s.validateSnapArchs([]*seedwriter.SeedSnap{sn}); err != nil {
-			return "", err
+			return "", nil, err
 		}
-		return sn.Path, nil
+
+		compPaths := make(map[string]string, len(cinfos))
+		for _, comp := range sn.Components {
+			compPaths[comp.ComponentName] = comp.Path
+		}
+
+		return sn.Path, compPaths, nil
 	}
 	snapToDownloadOptions := make([]tooling.SnapToDownload, len(snapsToDownload))
 	for i, sn := range snapsToDownload {
+		vss, rev, err := s.validationSetKeysAndRevisionForSnap(sn.SnapName())
+		if err != nil {
+			return nil, err
+		}
+
+		var channel string
+		switch {
+		case !rev.Unset():
+			// if we're setting a revision from a validation set, we don't want
+			// to send a channel, since we don't know if that revision is in
+			// that channel
+			channel = ""
+		case sn.Channel == "":
+			// otherwise, we want to make sure to set a default channel if
+			// possible. this case shouldn't ever really happen, since SeedSnaps
+			// should have a channel set
+			channel = "stable"
+		default:
+			channel = sn.Channel
+		}
+
 		byName[sn.SnapName()] = sn
+		revisions[sn.SnapName()] = rev
 		snapToDownloadOptions[i].Snap = sn
-		snapToDownloadOptions[i].Channel = sn.Channel
-		snapToDownloadOptions[i].Revision = s.revisions[sn.SnapName()]
+		snapToDownloadOptions[i].Channel = channel
+		snapToDownloadOptions[i].Revision = rev
 		snapToDownloadOptions[i].CohortKey = s.wideCohortKey
+		snapToDownloadOptions[i].ValidationSets = vss
+
+		// Components
+		compsToDownload := make([]string, len(sn.Components))
+		for i, comp := range sn.Components {
+			compsToDownload[i] = comp.ComponentRef.ComponentName
+		}
+		snapToDownloadOptions[i].CompsToDownload = compsToDownload
 	}
 
 	// sort the curSnaps slice for test consistency
@@ -506,6 +686,7 @@ func (s *imageSeeder) downloadSnaps(snapsToDownload []*seedwriter.SeedSnap, curS
 	if err != nil {
 		return nil, err
 	}
+
 	return downloadedSnaps, nil
 }
 
@@ -525,7 +706,7 @@ func localSnapsWithID(snaps localSnapRefs) []*tooling.CurrentSnap {
 	return localSnaps
 }
 
-func (s *imageSeeder) downloadAllSnaps(localSnaps localSnapRefs, fetchAsserts func(sn, sysSn, kernSn *seedwriter.SeedSnap) ([]*asserts.Ref, error)) error {
+func (s *imageSeeder) downloadAllSnaps(localSnaps localSnapRefs, fetchAsserts seedwriter.AssertsFetchFunc) error {
 	curSnaps := localSnapsWithID(localSnaps)
 	for {
 		toDownload, err := s.w.SnapsToDownload()
@@ -686,6 +867,17 @@ func (s *imageSeeder) warnOnUnassertedSnaps() error {
 }
 
 func (s *imageSeeder) finish() error {
+	// Ensure that the snapd snap is compatible with the snap-bootstrap
+	// contained within the kernel snap.
+	if err := s.w.VerifySnapBootstrapCompatibility(); err != nil {
+		if !s.allowSnapdKernelMismatch {
+			// If not, error out as there is no reason to allow
+			// this as the resulting image will be invalid.
+			return err
+		}
+		fmt.Fprintf(Stderr, "WARNING: %v\n", err)
+	}
+
 	// print any warnings that occurred during the download phase
 	for _, warn := range s.w.Warnings() {
 		fmt.Fprintf(Stderr, "WARNING: %s\n", warn)
@@ -696,6 +888,14 @@ func (s *imageSeeder) finish() error {
 		return err
 	}
 
+	// run validation-set checks, this is also done by store but
+	// we double-check for the seed.
+	if s.customizations.Validation != "ignore" {
+		if err := s.w.CheckValidationSets(); err != nil {
+			return err
+		}
+	}
+
 	copySnap := func(name, src, dst string) error {
 		fmt.Fprintf(Stdout, "Copying %q (%s)\n", src, name)
 		return osutil.CopyFile(src, dst, 0)
@@ -717,8 +917,20 @@ func (s *imageSeeder) finish() error {
 	return s.finishSeedCore()
 }
 
-func optionSnaps(opts *Options) []*seedwriter.OptionsSnap {
+func readComponentInfoFromCont(path string) (*snap.ComponentInfo, error) {
+	compf, err := snapfile.Open(path)
+	if err != nil {
+		return nil, fmt.Errorf("cannot open container: %w", err)
+	}
+
+	return snap.ReadComponentInfoFromContainer(compf, nil, nil)
+}
+
+func optionSnaps(opts *Options) ([]*seedwriter.OptionsSnap, []string, error) {
 	optSnaps := make([]*seedwriter.OptionsSnap, 0, len(opts.Snaps))
+	pathToLocalComp := map[string]*snap.ComponentInfo{}
+	localCompsPaths := []string{}
+
 	for _, snapName := range opts.Snaps {
 		var optSnap seedwriter.OptionsSnap
 		if strings.HasSuffix(snapName, ".snap") {
@@ -730,28 +942,43 @@ func optionSnaps(opts *Options) []*seedwriter.OptionsSnap {
 		optSnap.Channel = opts.SnapChannels[snapName]
 		optSnaps = append(optSnaps, &optSnap)
 	}
-	return optSnaps
-}
-
-// manifestFromLocalSnaps creates an initial seed manifest based on the locally
-// available snaps. It also performs initial verification against any rules given
-// to seedSetup against these.
-func manifestFromLocalSnaps(snaps localSnapRefs, opts *Options) (map[string]snap.Revision, error) {
-	seedManifest := make(map[string]snap.Revision)
-	for sn := range snaps {
-		// Its a bit more tricky to deal with local snaps, as we only have that specific revision
-		// available. Therefore the revision in the local snap must be exactly the revision specified
-		// in the manifest. If it's not, we fail.
-		specifiedRevision := opts.Revisions[sn.Info.SnapName()]
-		if !specifiedRevision.Unset() && specifiedRevision != sn.Info.Revision {
-			return nil, fmt.Errorf("cannot use snap %s for image, unknown/local revision does not match the value specified by revisions file (%s != %s)",
-				sn.Path, sn.Info.Revision, specifiedRevision)
-		}
-		if !sn.Info.Revision.Unset() {
-			seedManifest[sn.Info.SnapName()] = sn.Info.Revision
+	for _, compOpt := range opts.Components {
+		if strings.HasSuffix(compOpt, ".comp") {
+			// We need to look inside to know the owner snap, wait until
+			// that can be done for all local snaps/comps
+			cinfo, err := readComponentInfoFromCont(compOpt)
+			if err != nil {
+				return nil, nil, err
+			}
+			// Being a map, we ensure we do not get duplicates
+			pathToLocalComp[compOpt] = cinfo
+			localCompsPaths = append(localCompsPaths, compOpt)
+		} else {
+			snapName, compName, err := naming.SplitFullComponentName(compOpt)
+			if err != nil {
+				return nil, nil, err
+			}
+			optComp := seedwriter.OptionsComponent{Name: compName}
+			// Add the component to the matching snap, or create
+			// new otherwise (that is, assume that
+			// --comp <snap>+<comp> implicitly pulls also the snap)
+			snapFound := false
+			for _, optSn := range optSnaps {
+				if optSn.Name == snapName {
+					optSn.Components = append(optSn.Components, optComp)
+					snapFound = true
+					break
+				}
+			}
+			if !snapFound {
+				optSnaps = append(optSnaps, &seedwriter.OptionsSnap{
+					Name:       snapName,
+					Components: []seedwriter.OptionsComponent{optComp},
+				})
+			}
 		}
 	}
-	return seedManifest, nil
+	return optSnaps, localCompsPaths, nil
 }
 
 func selectAssertionMaxFormats(tsto *tooling.ToolingStore, model *asserts.Model, sysSn, kernSn *seedwriter.SeedSnap) error {
@@ -798,17 +1025,11 @@ var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opt
 		return err
 	}
 
-	// TODO: developer database in home or use snapd (but need
-	// a bit more API there, potential issues when crossing stores/series)
-	db, err := asserts.OpenDatabase(&asserts.DatabaseConfig{
-		Backstore: asserts.NewMemoryBackstore(),
-		Trusted:   trusted,
-	})
+	snapOpts, localCompsPaths, err := optionSnaps(opts)
 	if err != nil {
 		return err
 	}
-	f, err := s.start(db, optionSnaps(opts))
-	if err != nil {
+	if err := s.start(snapOpts); err != nil {
 		return err
 	}
 
@@ -817,20 +1038,12 @@ var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opt
 	// know the correct assertion max format to use.
 	// Fetch assertions tentatively into a temporary database
 	// and later either copy them or fetch more appropriate ones.
-	tmpDb := db.WithStackedBackstore(asserts.NewMemoryBackstore())
-	tmpFetcher := seedwriter.MakeRefAssertsFetcher(func(save func(asserts.Assertion) error) asserts.Fetcher {
+	tmpDb := s.db.WithStackedBackstore(asserts.NewMemoryBackstore())
+	tmpFetcher := seedwriter.MakeSeedAssertionFetcher(func(save func(asserts.Assertion) error) asserts.Fetcher {
 		return tsto.AssertionFetcher(tmpDb, save)
 	})
 
-	localSnaps, err := s.deriveInfoForLocalSnaps(tmpFetcher, tmpDb)
-	if err != nil {
-		return err
-	}
-
-	// Create the initial manifest, derived from the locally available snaps.
-	// Must be done after deriveInfoForLocalSnaps, as the snap info must have
-	// been derived if possible.
-	imageManifest, err := manifestFromLocalSnaps(localSnaps, opts)
+	localSnaps, err := s.deriveInfoForLocalSnaps(localCompsPaths, tmpFetcher, tmpDb)
 	if err != nil {
 		return err
 	}
@@ -857,12 +1070,12 @@ var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opt
 			}
 			if assertMaxFormats != nil && a.Format() > assertMaxFormats[aRef.Type.Name] {
 				// format was too new, re-fetch to replace
-				if err := f.Fetch(aRef); err != nil {
+				if err := s.f.Fetch(aRef); err != nil {
 					return err
 				}
 			} else {
 				// copy
-				if err := f.Save(a); err != nil {
+				if err := s.f.Save(a); err != nil {
 					return err
 				}
 			}
@@ -878,32 +1091,53 @@ var setupSeed = func(tsto *tooling.ToolingStore, model *asserts.Model, opts *Opt
 			assertMaxFormatsSelected = true
 			assertMaxFormats = tsto.AssertionMaxFormats()
 		}
-		prev := len(f.Refs())
+		prev := len(s.f.Refs())
 		if aRefs, ok := localSnaps[sn]; ok {
 			if err := copyOrRefetchIfFormatTooNewIntoDb(aRefs); err != nil {
 				return nil, err
 			}
 		} else {
-			// fetch snap assertions
-			if _, err = FetchAndCheckSnapAssertions(sn.Path, sn.Info, model, f, db); err != nil {
+			// fetch snap and components assertions
+			compPaths := make([]CompInfoPath, len(sn.Components))
+			for i, comp := range sn.Components {
+				compPaths[i] = CompInfoPath{
+					Info: comp.Info,
+					Path: comp.Path,
+				}
+			}
+			if _, err = FetchAndCheckSnapAssertions(sn.Path, sn.Info, compPaths, model, s.f, s.db); err != nil {
 				return nil, err
 			}
-			if !sn.Info.Revision.Unset() {
-				imageManifest[sn.Info.SnapName()] = sn.Info.Revision
-			}
 		}
-		return f.Refs()[prev:], nil
+		return s.f.Refs()[prev:], nil
 	}
 
 	if err := s.downloadAllSnaps(localSnaps, fetchAsserts); err != nil {
 		return err
 	}
-
-	// last thing is to generate the image seed manifest file
-	if opts.SeedManifestPath != "" {
-		if err := WriteSeedManifest(opts.SeedManifestPath, imageManifest); err != nil {
-			return err
-		}
-	}
 	return s.finish()
 }
+
+func decodeExtraAssertions(r io.Reader) ([]asserts.Assertion, error) {
+	var extraAssertions []asserts.Assertion
+
+	dec := asserts.NewDecoder(r)
+	for {
+		a, err := dec.Decode()
+		if err == io.EOF {
+			break
+		}
+		if err != nil {
+			return nil, fmt.Errorf("failed to decode extra assertion: %v", err)
+		}
+
+		switch a.Type() {
+		case asserts.SnapDeclarationType, asserts.SnapRevisionType, asserts.ModelType, asserts.SerialType, asserts.ValidationSetType:
+			return nil, fmt.Errorf("assertion type %v is not allowed for extra assertions", a.Type().Name)
+		}
+
+		extraAssertions = append(extraAssertions, a)
+	}
+
+	return extraAssertions, nil
+}
