@@ -71,6 +71,8 @@ network packet,
 @{PROC}/sys/net/netfilter/ r,
 @{PROC}/sys/net/netfilter/** rw,
 @{PROC}/sys/net/nf_conntrack_max rw,
+@{PROC}/sys/net/mptcp/ r,
+@{PROC}/sys/net/mptcp/** rw,
 
 # Needed for systemd's dhcp implementation
 @{PROC}/sys/kernel/random/boot_id r,
@@ -88,6 +90,9 @@ network packet,
 
 /run/udev/data/* r,
 
+# Allow access to read the gnutls config
+/etc/gnutls/config r,
+
 # Allow read and write access for all netplan configuration files
 # as NetworkManager will start using them to store the network
 # configuration instead of using its own internal keyfile based
@@ -152,6 +157,20 @@ dbus (send)
      member="SetLink{DefaultRoute,DNSOverTLS,DNS,DNSEx,DNSSEC,DNSSECNegativeTrustAnchors,MulticastDNS,Domains,LLMNR}"
      peer=(label=unconfined),
 
+dbus (send)
+     bus=system
+     path="/org/freedesktop/resolve1"
+     interface="org.freedesktop.resolve1.Manager"
+     member="GetLink"
+     peer=(name="org.freedesktop.resolve1", label=unconfined),
+
+dbus (send)
+     bus=system
+     path="/org/freedesktop/resolve1/link/*"
+     interface="org.freedesktop.resolve1.Link"
+     member="Set{DNS,DNSSEC,DNSSECNegativeTrustAnchors,MulticastDNS,Domains,LLMNR}"
+     peer=(name="org.freedesktop.resolve1", label=unconfined),
+
 dbus (send)
    bus=system
    path=/org/freedesktop/DBus
@@ -295,6 +314,9 @@ const networkManagerConnectedPlugAppArmor = `
 
 #include <abstractions/dbus-strict>
 
+# Allow access to gnutls config
+/etc/gnutls/config r,
+
 # Allow all access to NetworkManager service
 dbus (receive, send)
     bus=system
@@ -526,7 +548,7 @@ func (iface *networkManagerInterface) AppArmorConnectedPlug(spec *apparmor.Speci
 		// of the OS snap and will run unconfined.
 		new = "unconfined"
 	} else {
-		new = spec.SnapAppSet().SlotLabelExpression(slot)
+		new = slot.LabelExpression()
 	}
 	snippet := strings.Replace(networkManagerConnectedPlugAppArmor, old, new, -1)
 	spec.AddSnippet(snippet)
@@ -540,7 +562,7 @@ func (iface *networkManagerInterface) AppArmorConnectedPlug(spec *apparmor.Speci
 
 func (iface *networkManagerInterface) AppArmorConnectedSlot(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	old := "###PLUG_SECURITY_TAGS###"
-	new := spec.SnapAppSet().PlugLabelExpression(plug)
+	new := plug.LabelExpression()
 	snippet := strings.Replace(networkManagerConnectedSlotAppArmor, old, new, -1)
 	spec.AddSnippet(snippet)
 	if !release.OnClassic {
