@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016-2022 Canonical Ltd
+ * Copyright (C) 2016-2024 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -30,6 +30,7 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/features"
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/interfaces/policy"
@@ -37,6 +38,7 @@ import (
 	"github.com/snapcore/snapd/jsonutil"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/overlord/assertstate"
+	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/ifacestate/schema"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
@@ -51,14 +53,13 @@ func init() {
 
 var (
 	snapdAppArmorServiceIsDisabled = snapdAppArmorServiceIsDisabledImpl
-	profilesNeedRegeneration       = profilesNeedRegenerationImpl
 
 	writeSystemKey = interfaces.WriteSystemKey
 )
 
-func (m *InterfaceManager) selectInterfaceMapper(snaps []*snap.Info) {
-	for _, snapInfo := range snaps {
-		if snapInfo.Type() == snap.TypeSnapd {
+func (m *InterfaceManager) selectInterfaceMapper(appSets []*interfaces.SnapAppSet) {
+	for _, set := range appSets {
+		if set.Info().Type() == snap.TypeSnapd {
 			mapper = &CoreSnapdSystemMapper{}
 			break
 		}
@@ -79,18 +80,18 @@ func (m *InterfaceManager) addInterfaces(extra []interfaces.Interface) error {
 	return nil
 }
 
-func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error {
+func (m *InterfaceManager) securityBackendOpts() (*interfaces.SecurityBackendOptions, error) {
 	// get the snapd snap info if it is installed
 	var snapdSnap snapstate.SnapState
 	var snapdSnapInfo *snap.Info
 	err := snapstate.Get(m.state, "snapd", &snapdSnap)
 	if err != nil && !errors.Is(err, state.ErrNoState) {
-		return fmt.Errorf("cannot access snapd snap state: %v", err)
+		return nil, fmt.Errorf("cannot access snapd snap state: %w", err)
 	}
 	if err == nil {
 		snapdSnapInfo, err = snapdSnap.CurrentInfo()
 		if err != nil && err != snapstate.ErrNoCurrent {
-			return fmt.Errorf("cannot access snapd snap info: %v", err)
+			return nil, fmt.Errorf("cannot access snapd snap info: %w", err)
 		}
 	}
 
@@ -99,12 +100,12 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 	var coreSnapInfo *snap.Info
 	err = snapstate.Get(m.state, "core", &coreSnap)
 	if err != nil && !errors.Is(err, state.ErrNoState) {
-		return fmt.Errorf("cannot access core snap state: %v", err)
+		return nil, fmt.Errorf("cannot access core snap state: %w", err)
 	}
 	if err == nil {
 		coreSnapInfo, err = coreSnap.CurrentInfo()
 		if err != nil && err != snapstate.ErrNoCurrent {
-			return fmt.Errorf("cannot access core snap info: %v", err)
+			return nil, fmt.Errorf("cannot access core snap info: %w", err)
 		}
 	}
 
@@ -113,8 +114,18 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 		CoreSnapInfo:  coreSnapInfo,
 		SnapdSnapInfo: snapdSnapInfo,
 	}
+
+	return &opts, nil
+}
+
+func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error {
+	opts, err := m.securityBackendOpts()
+	if err != nil {
+		return err
+	}
+
 	for _, backend := range allSecurityBackends() {
-		if err := backend.Initialize(&opts); err != nil {
+		if err := backend.Initialize(opts); err != nil {
 			return err
 		}
 		if err := m.repo.AddBackend(backend); err != nil {
@@ -122,7 +133,7 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 		}
 	}
 	for _, backend := range extra {
-		if err := backend.Initialize(&opts); err != nil {
+		if err := backend.Initialize(opts); err != nil {
 			return err
 		}
 		if err := m.repo.AddBackend(backend); err != nil {
@@ -132,20 +143,49 @@ func (m *InterfaceManager) addBackends(extra []interfaces.SecurityBackend) error
 	return nil
 }
 
-func (m *InterfaceManager) addSnaps(snaps []*snap.Info) error {
-	for _, snapInfo := range snaps {
-		if err := addImplicitSlots(m.state, snapInfo); err != nil {
-			return err
+// Reinitializes compatible backends which have previously been added to the
+// repository.
+func (m *InterfaceManager) reinitializeBackends(tm timings.Measurer) error {
+	for _, b := range m.repo.Backends() {
+		rb, ok := b.(interfaces.ReinitializableSecurityBackend)
+		if !ok {
+			continue
 		}
-		if err := m.repo.AddSnap(snapInfo); err != nil {
-			logger.Noticef("cannot add snap %q to interface repository: %s", snapInfo.InstanceName(), err)
+
+		var err error
+		timings.Run(tm, "reinitialize-security-backend", fmt.Sprintf("reinitialize %q security backend", b.Name()),
+			func(nesttm timings.Measurer) {
+				err = rb.Reinitialize()
+			})
+		if err != nil {
+			return fmt.Errorf("cannot reinitialize backend %q: %w", b.Name(), err)
 		}
 	}
 	return nil
 }
 
-func profilesNeedRegenerationImpl() bool {
-	mismatch, err := interfaces.SystemKeyMismatch()
+func (m *InterfaceManager) addAppSets(appSets []*interfaces.SnapAppSet) error {
+	for _, set := range appSets {
+		if err := addImplicitInterfaces(m.state, set.Info()); err != nil {
+			return err
+		}
+
+		if err := m.repo.AddAppSet(set); err != nil {
+			logger.Noticef("cannot add app set for snap %q to interface repository: %s", set.Info().InstanceName(), err)
+		}
+	}
+	return nil
+}
+
+func (m *InterfaceManager) profilesNeedRegeneration() bool {
+	return profilesNeedRegenerationImpl(m)
+}
+
+var profilesNeedRegenerationImpl = func(m *InterfaceManager) bool {
+	extraData := interfaces.SystemKeyExtraData{
+		AppArmorPrompting: m.useAppArmorPrompting,
+	}
+	mismatch, _, err := interfaces.SystemKeyMismatch(extraData)
 	if err != nil {
 		logger.Noticef("error trying to compare the snap system key: %v", err)
 		return true
@@ -153,6 +193,17 @@ func profilesNeedRegenerationImpl() bool {
 	return mismatch
 }
 
+// Checks whether AppArmor Prompting should be used. Caller must lock m.state.
+func (m *InterfaceManager) assessAppArmorPrompting() bool {
+	tr := config.NewTransaction(m.state)
+	if promptingEnabled, err := features.Flag(tr, features.AppArmorPrompting); err == nil {
+		supported, _ := features.AppArmorPrompting.IsSupported()
+		// If error while getting AppArmorPrompting flag, don't include it
+		return promptingEnabled && supported
+	}
+	return false
+}
+
 // snapdAppArmorServiceIsDisabledImpl returns true if the snapd.apparmor
 // service unit exists but is disabled
 func snapdAppArmorServiceIsDisabledImpl() bool {
@@ -161,28 +212,51 @@ func snapdAppArmorServiceIsDisabledImpl() bool {
 	return err == nil && !isEnabled
 }
 
-// regenerateAllSecurityProfiles will regenerate all security profiles.
-func (m *InterfaceManager) regenerateAllSecurityProfiles(tm timings.Measurer) error {
+// regenerateAllSecurityProfiles will regenerate all security profiles. This
+// function is expected to be called with the state locked, though in some
+// scenarios one may want to temporarily unlock the state for the duration of
+// security backends executing their setup.
+func (m *InterfaceManager) regenerateAllSecurityProfiles(tm timings.Measurer, unlockState bool) error {
 	// Get all the security backends
 	securityBackends := m.repo.Backends()
 
 	// Get all the snap infos
-	snaps, err := snapsWithSecurityProfiles(m.state)
+	appSets, err := snapsWithSecurityProfiles(m.state)
 	if err != nil {
 		return err
 	}
 
-	// TODO: should snapsWithSecurityProfiles return app sets instead of snap infos?
-	appSets := make([]*interfaces.SnapAppSet, 0, len(snaps))
-	for _, sn := range snaps {
-		appSets = append(appSets, interfaces.NewSnapAppSet(sn))
+	precompOpts := make(map[string]interfaces.ConfinementOptions, len(appSets))
+
+	computeConfinementOpts := func(instanceName string) (interfaces.ConfinementOptions, error) {
+		var snapst snapstate.SnapState
+		if err := snapstate.Get(m.state, instanceName, &snapst); err != nil {
+			return interfaces.ConfinementOptions{}, err
+		}
+		snapInfo, err := snapst.CurrentInfo()
+		if err != nil {
+			return interfaces.ConfinementOptions{}, err
+		}
+		opts, err := m.buildConfinementOptions(m.state, nil, snapInfo, snapst.Flags)
+		if err != nil {
+			return interfaces.ConfinementOptions{}, err
+		}
+		return opts, nil
 	}
 
-	// Add implicit slots to all snaps
-	for _, snapInfo := range snaps {
-		if err := addImplicitSlots(m.state, snapInfo); err != nil {
+	for _, set := range appSets {
+		if err := addImplicitInterfaces(m.state, set.Info()); err != nil {
 			return err
 		}
+
+		instanceName := set.InstanceName()
+		optsForAppSet, err := computeConfinementOpts(instanceName)
+		if err != nil {
+			logger.Noticef("cannot get confinement options for snap %q: %v", instanceName, err)
+			continue
+		}
+
+		precompOpts[instanceName] = optsForAppSet
 	}
 
 	// The reason the system key is unlinked is to prevent snapd from believing
@@ -195,40 +269,37 @@ func (m *InterfaceManager) regenerateAllSecurityProfiles(tm timings.Measurer) er
 	shouldWriteSystemKey := true
 	os.Remove(dirs.SnapSystemKeyFile)
 
-	confinementOpts := func(snapName string) interfaces.ConfinementOptions {
-		var snapst snapstate.SnapState
-		if err := snapstate.Get(m.state, snapName, &snapst); err != nil {
-			logger.Noticef("cannot get state of snap %q: %s", snapName, err)
-			return interfaces.ConfinementOptions{}
-		}
-		snapInfo, err := snapst.CurrentInfo()
-		if err != nil {
-			logger.Noticef("cannot get current info for snap %q: %s", snapName, err)
-			return interfaces.ConfinementOptions{}
-		}
-		opts, err := buildConfinementOptions(m.state, snapInfo, snapst.Flags)
-		if err != nil {
-			logger.Noticef("cannot get confinement options for snap %q: %s", snapName, err)
-		}
-		return opts
+	precomputedConfinementOpts := func(instanceName string) interfaces.ConfinementOptions {
+		// options or default zero value
+		return precompOpts[instanceName]
 	}
 
-	// For each backend:
-	for _, backend := range securityBackends {
-		if backend.Name() == "" {
-			continue // Test backends have no name, skip them to simplify testing.
+	func() {
+		if unlockState {
+			m.state.Unlock()
+			defer m.state.Lock()
 		}
-		if errors := interfaces.SetupMany(m.repo, backend, appSets, confinementOpts, tm); len(errors) > 0 {
-			logger.Noticef("cannot regenerate %s profiles", backend.Name())
-			for _, err := range errors {
-				logger.Noticef(err.Error())
+
+		// For each backend:
+		for _, backend := range securityBackends {
+			if backend.Name() == "" {
+				continue // Test backends have no name, skip them to simplify testing.
+			}
+			if errors := interfaces.SetupMany(m.repo, backend, appSets, precomputedConfinementOpts, tm); len(errors) > 0 {
+				logger.Noticef("cannot regenerate %s profiles", backend.Name())
+				for _, err := range errors {
+					logger.Notice(err.Error())
+				}
+				shouldWriteSystemKey = false
 			}
-			shouldWriteSystemKey = false
 		}
-	}
+	}()
 
 	if shouldWriteSystemKey {
-		if err := writeSystemKey(); err != nil {
+		extraData := interfaces.SystemKeyExtraData{
+			AppArmorPrompting: m.useAppArmorPrompting,
+		}
+		if err := writeSystemKey(extraData); err != nil {
 			logger.Noticef("cannot write system key: %v", err)
 		}
 	}
@@ -331,6 +402,28 @@ func (m *InterfaceManager) reloadConnections(snapName string) ([]string, error)
 		return nil, err
 	}
 
+	var policyChecker interfaces.PolicyFunc
+	var autoChecker *autoConnectChecker
+	var connChecker *connectChecker
+
+	deviceCtx, err := snapstate.DeviceCtx(m.state, nil, nil)
+	if errors.Is(err, state.ErrNoState) {
+		// everything else is a noop, as no model means no connections
+		// to reload
+		return nil, nil
+	} else if err != nil {
+		return nil, err
+	}
+	autoChecker, err = newAutoConnectChecker(m.state, m.repo, deviceCtx)
+	if err != nil {
+		return nil, err
+	}
+
+	connChecker, err = newConnectChecker(m.state, deviceCtx)
+	if err != nil {
+		return nil, err
+	}
+
 	connStateChanged := false
 	affected := make(map[string]bool)
 ConnsLoop:
@@ -385,27 +478,48 @@ ConnsLoop:
 		var updateStaticAttrs bool
 		staticPlugAttrs := connState.StaticPlugAttrs
 		staticSlotAttrs := connState.StaticSlotAttrs
+		newStaticPlugAttrs := utils.NormalizeInterfaceAttributes(plugInfo.Attrs).(map[string]any)
+		newStaticSlotAttrs := utils.NormalizeInterfaceAttributes(slotInfo.Attrs).(map[string]any)
 
-		// XXX: Refresh the copy of the static connection attributes for "content"
-		// and "system-files" interfaces.
-		// This is a partial and temporary solution to https://bugs.launchpad.net/snapd/+bug/1825883
-		// and https://bugs.launchpad.net/snapd/+bug/1942266.
-		switch plugInfo.Interface {
-		case "content":
-			var plugContent, slotContent string
-			plugInfo.Attr("content", &plugContent)
-			slotInfo.Attr("content", &slotContent)
+		// if the interface was originally autoconnected, update the static attrs if it would
+		// still be allowed to autoconnect. Otherwise, update the static attrs if it would still
+		// be allowed to regular connect.
+		if connState.Auto && !connState.ByGadget {
+			policyChecker = func(cplug *interfaces.ConnectedPlug, cslot *interfaces.ConnectedSlot) (bool, error) {
+				iface, err := interfaces.ByName(cplug.Interface())
+				if err != nil {
+					return false, err
+				}
 
-			if plugContent != "" && plugContent == slotContent {
-				staticPlugAttrs = utils.NormalizeInterfaceAttributes(plugInfo.Attrs).(map[string]interface{})
-				staticSlotAttrs = utils.NormalizeInterfaceAttributes(slotInfo.Attrs).(map[string]interface{})
-				updateStaticAttrs = true
-			} else {
-				logger.Noticef("cannot refresh static attributes of the connection %q", connId)
+				if !iface.AutoConnect(plugInfo, slotInfo) {
+					return false, nil
+				}
+
+				ok, _, err := autoChecker.check(cplug, cslot)
+				return ok, err
 			}
-		case "system-files":
-			staticPlugAttrs = utils.NormalizeInterfaceAttributes(plugInfo.Attrs).(map[string]interface{})
-			staticSlotAttrs = utils.NormalizeInterfaceAttributes(slotInfo.Attrs).(map[string]interface{})
+		} else {
+			policyChecker = connChecker.check
+		}
+
+		plugAppSet, err := interfaces.NewSnapAppSet(plugInfo.Snap, nil)
+		if err != nil {
+			return nil, err
+		}
+		slotAppSet, err := interfaces.NewSnapAppSet(slotInfo.Snap, nil)
+		if err != nil {
+			return nil, err
+		}
+
+		cplug := interfaces.NewConnectedPlug(plugInfo, plugAppSet, newStaticPlugAttrs, connState.DynamicPlugAttrs)
+		cslot := interfaces.NewConnectedSlot(slotInfo, slotAppSet, newStaticSlotAttrs, connState.DynamicSlotAttrs)
+
+		ok, err := policyChecker(cplug, cslot)
+		if !ok || err != nil {
+			logger.Noticef("cannot refresh static attributes of the connection %q", connId)
+		} else {
+			staticPlugAttrs = newStaticPlugAttrs
+			staticSlotAttrs = newStaticSlotAttrs
 			updateStaticAttrs = true
 		}
 
@@ -687,7 +801,6 @@ var DebugAutoConnectCheck func(*policy.ConnectCandidate, interfaces.SideArity, e
 
 type autoConnectChecker struct {
 	st   *state.State
-	task *state.Task
 	repo *interfaces.Repository
 
 	deviceCtx snapstate.DeviceContext
@@ -695,14 +808,13 @@ type autoConnectChecker struct {
 	baseDecl  *asserts.BaseDeclaration
 }
 
-func newAutoConnectChecker(s *state.State, task *state.Task, repo *interfaces.Repository, deviceCtx snapstate.DeviceContext) (*autoConnectChecker, error) {
+func newAutoConnectChecker(s *state.State, repo *interfaces.Repository, deviceCtx snapstate.DeviceContext) (*autoConnectChecker, error) {
 	baseDecl, err := assertstate.BaseDeclaration(s)
 	if err != nil {
 		return nil, fmt.Errorf("internal error: cannot find base declaration: %v", err)
 	}
 	return &autoConnectChecker{
 		st:        s,
-		task:      task,
 		repo:      repo,
 		deviceCtx: deviceCtx,
 		cache:     make(map[string]*asserts.SnapDeclaration),
@@ -816,7 +928,7 @@ func filterUbuntuCoreSlots(candidates []*snap.SlotInfo, arities []interfaces.Sid
 // conns. cannotAutoConnectLog is called to build a log message in
 // case no applicable pair was found. conflictError is called
 // to handle checkAutoconnectConflicts errors.
-func (c *autoConnectChecker) addAutoConnections(newconns map[string]*interfaces.ConnRef, plugs []*snap.PlugInfo, filter func([]*snap.SlotInfo) []*snap.SlotInfo, conns map[string]*schema.ConnState, cannotAutoConnectLog func(plug *snap.PlugInfo, candRefs []string) string, conflictError func(*state.Retry, error) error) error {
+func (c *autoConnectChecker) addAutoConnections(task *state.Task, newconns map[string]*interfaces.ConnRef, plugs []*snap.PlugInfo, filter func([]*snap.SlotInfo) []*snap.SlotInfo, conns map[string]*schema.ConnState, cannotAutoConnectLog func(plug *snap.PlugInfo, candRefs []string) string, conflictError func(*state.Retry, error) error) error {
 	for _, plug := range plugs {
 		candSlots, arities := c.repo.AutoConnectCandidateSlots(plug.Snap.InstanceName(), plug.Name, c.check)
 
@@ -852,12 +964,12 @@ func (c *autoConnectChecker) addAutoConnections(newconns map[string]*interfaces.
 			for i, candidate := range candSlots {
 				crefs[i] = candidate.String()
 			}
-			c.task.Logf(cannotAutoConnectLog(plug, crefs))
+			task.Logf(cannotAutoConnectLog(plug, crefs))
 			continue
 		}
 
 		for _, slot := range applicable {
-			if err := addNewConnection(c.st, c.task, newconns, conns, plug, slot, conflictError); err != nil {
+			if err := addNewConnection(c.st, task, newconns, conns, plug, slot, conflictError); err != nil {
 				return err
 			}
 		}
@@ -974,10 +1086,10 @@ func getConns(st *state.State) (conns map[string]*schema.ConnState, err error) {
 		}
 		cref.PlugRef.Snap = RemapSnapFromState(cref.PlugRef.Snap)
 		cref.SlotRef.Snap = RemapSnapFromState(cref.SlotRef.Snap)
-		cstate.StaticSlotAttrs = utils.NormalizeInterfaceAttributes(cstate.StaticSlotAttrs).(map[string]interface{})
-		cstate.DynamicSlotAttrs = utils.NormalizeInterfaceAttributes(cstate.DynamicSlotAttrs).(map[string]interface{})
-		cstate.StaticPlugAttrs = utils.NormalizeInterfaceAttributes(cstate.StaticPlugAttrs).(map[string]interface{})
-		cstate.DynamicPlugAttrs = utils.NormalizeInterfaceAttributes(cstate.DynamicPlugAttrs).(map[string]interface{})
+		cstate.StaticSlotAttrs = utils.NormalizeInterfaceAttributes(cstate.StaticSlotAttrs).(map[string]any)
+		cstate.DynamicSlotAttrs = utils.NormalizeInterfaceAttributes(cstate.DynamicSlotAttrs).(map[string]any)
+		cstate.StaticPlugAttrs = utils.NormalizeInterfaceAttributes(cstate.StaticPlugAttrs).(map[string]any)
+		cstate.DynamicPlugAttrs = utils.NormalizeInterfaceAttributes(cstate.DynamicPlugAttrs).(map[string]any)
 		remapped[cref.ID()] = cstate
 	}
 	return remapped, nil
@@ -1007,12 +1119,12 @@ func setConns(st *state.State, conns map[string]*schema.ConnState) {
 // is tracked with SnapState.PendingSecurity,
 // or snap about to be active (pending link-snap) with a done
 // setup-profiles
-func snapsWithSecurityProfiles(st *state.State) ([]*snap.Info, error) {
+func snapsWithSecurityProfiles(st *state.State) ([]*interfaces.SnapAppSet, error) {
 	all, err := snapstate.All(st)
 	if err != nil {
 		return nil, err
 	}
-	infos := make([]*snap.Info, 0, len(all))
+	appSets := make([]*interfaces.SnapAppSet, 0, len(all))
 	seen := make(map[string]bool, len(all))
 	for instanceName, snapst := range all {
 		if snapst.Active {
@@ -1021,7 +1133,14 @@ func snapsWithSecurityProfiles(st *state.State) ([]*snap.Info, error) {
 				logger.Noticef("cannot retrieve info for snap %q: %s", instanceName, err)
 				continue
 			}
-			infos = append(infos, snapInfo)
+
+			set, err := appSetForSnapRevision(st, snapInfo)
+			if err != nil {
+				logger.Noticef("cannot build app set for snap %q: %s", instanceName, err)
+				continue
+			}
+
+			appSets = append(appSets, set)
 			seen[instanceName] = true
 		} else if snapst.PendingSecurity != nil {
 			// we tracked any pending security profiles for the snap
@@ -1036,9 +1155,28 @@ func snapsWithSecurityProfiles(st *state.State) ([]*snap.Info, error) {
 				logger.Noticef("cannot retrieve info for snap %q: %s", instanceName, err)
 				continue
 			}
-			infos = append(infos, snapInfo)
+
+			components := make([]*snap.ComponentInfo, 0, len(snapst.PendingSecurity.Components))
+			for _, csi := range snapst.PendingSecurity.Components {
+				ci, err := snapstate.ReadComponentInfo(snapInfo, csi)
+				if err != nil {
+					logger.Noticef("cannot read component info for snap %q: %s", instanceName, err)
+					continue
+				}
+
+				components = append(components, ci)
+			}
+
+			set, err := interfaces.NewSnapAppSet(snapInfo, components)
+			if err != nil {
+				logger.Noticef("cannot build app set for snap %q: %s", instanceName, err)
+				continue
+			}
+
+			appSets = append(appSets, set)
 		}
 	}
+
 	// look at the changes for old snapds and also
 	// the situation that are being installed, so they do not
 	// have SnapState yet
@@ -1078,10 +1216,19 @@ func snapsWithSecurityProfiles(st *state.State) ([]*snap.Info, error) {
 			logger.Noticef("cannot retrieve info for snap %q: %s", instanceName, err)
 			continue
 		}
-		infos = append(infos, snapInfo)
+
+		// this should find any component setups that exist on the task and add
+		// them to the app set
+		set, err := appSetForTask(t, snapInfo)
+		if err != nil {
+			logger.Noticef("cannot build app set for snap %q: %s", instanceName, err)
+			continue
+		}
+
+		appSets = append(appSets, set)
 	}
 
-	return infos, nil
+	return appSets, nil
 }
 
 func resolveSnapIDToName(st *state.State, snapID string) (name string, err error) {
@@ -1316,10 +1463,10 @@ func addHotplugSeqWaitTask(hotplugChange *state.Change, hotplugKey snap.HotplugK
 }
 
 type HotplugSlotInfo struct {
-	Name        string                 `json:"name"`
-	Interface   string                 `json:"interface"`
-	StaticAttrs map[string]interface{} `json:"static-attrs,omitempty"`
-	HotplugKey  snap.HotplugKey        `json:"hotplug-key"`
+	Name        string          `json:"name"`
+	Interface   string          `json:"interface"`
+	StaticAttrs map[string]any  `json:"static-attrs,omitempty"`
+	HotplugKey  snap.HotplugKey `json:"hotplug-key"`
 
 	// device was unplugged but has connections, so slot is remembered
 	HotplugGone bool `json:"hotplug-gone"`
@@ -1388,3 +1535,66 @@ func hasActiveConnection(st *state.State, iface string) (bool, error) {
 	}
 	return false, nil
 }
+
+func appSetForTask(t *state.Task, info *snap.Info) (*interfaces.SnapAppSet, error) {
+	compsups, err := snapstate.ComponentSetupsForTask(t)
+	if err != nil {
+		return nil, err
+	}
+
+	compInfos := make([]*snap.ComponentInfo, 0, len(compsups))
+	for _, compsup := range compsups {
+		compInfo, err := snapstate.ComponentInfoFromComponentSetup(compsup, info)
+		if err != nil {
+			return nil, err
+		}
+		compInfos = append(compInfos, compInfo)
+	}
+
+	st := t.State()
+
+	var snapst snapstate.SnapState
+	if err := snapstate.Get(st, info.InstanceName(), &snapst); err != nil {
+		// if the snap isn't in the state, then we know that there aren't any
+		// pre-existing components to consider
+		if errors.Is(err, state.ErrNoState) {
+			return interfaces.NewSnapAppSet(info, compInfos)
+		}
+		return nil, err
+	}
+
+	// if we're installing/refreshing a component then we need to consider the
+	// components that are already installed
+	if snapst.LastIndex(info.Revision) != -1 {
+		compsForRevision, err := snapst.ComponentInfosForRevision(info.Revision)
+		if err != nil {
+			return nil, err
+		}
+		compInfos = append(compInfos, compsForRevision...)
+	}
+
+	return interfaces.NewSnapAppSet(info, compInfos)
+}
+
+func appSetForSnapRevision(st *state.State, info *snap.Info) (*interfaces.SnapAppSet, error) {
+	var snapst snapstate.SnapState
+	if err := snapstate.Get(st, info.InstanceName(), &snapst); err != nil {
+		return nil, err
+	}
+
+	compInfos, err := snapst.ComponentInfosForRevision(info.Revision)
+	if err != nil {
+		return nil, err
+	}
+
+	return interfaces.NewSnapAppSet(info, compInfos)
+}
+
+func hasAppArmorBackend(backends []interfaces.SecurityBackend) bool {
+	for _, b := range backends {
+		if b.Name() == interfaces.SecurityAppArmor {
+			return true
+		}
+	}
+	return false
+}
