@@ -20,9 +20,11 @@
 package snapstate_test
 
 import (
+	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
+	"reflect"
 	"time"
 
 	. "gopkg.in/check.v1"
@@ -34,9 +36,12 @@ import (
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/snapstate"
+	"github.com/snapcore/snapd/overlord/snapstate/backend"
+	"github.com/snapcore/snapd/overlord/snapstate/sequence"
 	"github.com/snapcore/snapd/overlord/snapstate/snapstatetest"
 	"github.com/snapcore/snapd/overlord/state"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/testutil"
 )
 
@@ -231,8 +236,11 @@ func (s *snapmgrTestSuite) TestRemoveDiskSpaceForSnapshotError(c *C) {
 }
 
 func (s *snapmgrTestSuite) TestRemoveRunThrough(c *C) {
-	c.Assert(snapstate.KeepAuxStoreInfo("some-snap-id", nil), IsNil)
-	c.Check(snapstate.AuxStoreInfoFilename("some-snap-id"), testutil.FilePresent)
+	aux := backend.AuxStoreInfo{}    // doesn't matter for this test
+	linkCtx := backend.LinkContext{} // doesn't matter for this test
+	_, err := backend.InstallStoreMetadata("some-snap-id", aux, linkCtx)
+	c.Check(err, IsNil)
+	c.Check(backend.AuxStoreInfoFilename("some-snap-id"), testutil.FilePresent)
 	si := snap.SideInfo{
 		SnapID:   "some-snap-id",
 		RealName: "some-snap",
@@ -369,7 +377,7 @@ func (s *snapmgrTestSuite) TestRemoveRunThrough(c *C) {
 	var snapst snapstate.SnapState
 	err = snapstate.Get(s.state, "some-snap", &snapst)
 	c.Assert(err, testutil.ErrorIs, state.ErrNoState)
-	c.Check(snapstate.AuxStoreInfoFilename("some-snap-id"), testutil.FileAbsent)
+	c.Check(backend.AuxStoreInfoFilename("some-snap-id"), testutil.FileAbsent)
 
 }
 
@@ -415,8 +423,9 @@ func (s *snapmgrTestSuite) TestParallelInstanceRemoveRunThrough(c *C) {
 			name: "some-snap_instance",
 		},
 		{
-			op:   "unlink-snap",
-			path: filepath.Join(dirs.SnapMountDir, "some-snap_instance/7"),
+			op:             "unlink-snap",
+			path:           filepath.Join(dirs.SnapMountDir, "some-snap_instance/7"),
+			otherInstances: true,
 		},
 		{
 			op:    "remove-profiles:Doing",
@@ -438,7 +447,7 @@ func (s *snapmgrTestSuite) TestParallelInstanceRemoveRunThrough(c *C) {
 		{
 			op:             "remove-snap-data-dir",
 			name:           "some-snap_instance",
-			path:           filepath.Join(dirs.SnapDataDir, "some-snap"),
+			path:           filepath.Join(dirs.SnapDataDir, "some-snap_instance"),
 			otherInstances: true,
 		},
 		{
@@ -461,7 +470,7 @@ func (s *snapmgrTestSuite) TestParallelInstanceRemoveRunThrough(c *C) {
 		{
 			op:             "remove-snap-dir",
 			name:           "some-snap_instance",
-			path:           filepath.Join(dirs.SnapMountDir, "some-snap"),
+			path:           filepath.Join(dirs.SnapMountDir, "some-snap_instance"),
 			otherInstances: true,
 		},
 	}
@@ -569,8 +578,9 @@ func (s *snapmgrTestSuite) TestParallelInstanceRemoveRunThroughOtherInstances(c
 			name: "some-snap_instance",
 		},
 		{
-			op:   "unlink-snap",
-			path: filepath.Join(dirs.SnapMountDir, "some-snap_instance/7"),
+			op:             "unlink-snap",
+			path:           filepath.Join(dirs.SnapMountDir, "some-snap_instance/7"),
+			otherInstances: true,
 		},
 		{
 			op:    "remove-profiles:Doing",
@@ -592,7 +602,7 @@ func (s *snapmgrTestSuite) TestParallelInstanceRemoveRunThroughOtherInstances(c
 		{
 			op:             "remove-snap-data-dir",
 			name:           "some-snap_instance",
-			path:           filepath.Join(dirs.SnapDataDir, "some-snap"),
+			path:           filepath.Join(dirs.SnapDataDir, "some-snap_instance"),
 			otherInstances: true,
 		},
 		{
@@ -615,7 +625,7 @@ func (s *snapmgrTestSuite) TestParallelInstanceRemoveRunThroughOtherInstances(c
 		{
 			op:             "remove-snap-dir",
 			name:           "some-snap_instance",
-			path:           filepath.Join(dirs.SnapMountDir, "some-snap"),
+			path:           filepath.Join(dirs.SnapMountDir, "some-snap_instance"),
 			otherInstances: true,
 		},
 	}
@@ -1042,7 +1052,10 @@ func (s *snapmgrTestSuite) TestRemoveLastRevisionRunThrough(c *C) {
 	c.Assert(err, testutil.ErrorIs, state.ErrNoState)
 }
 
-func (s *snapmgrTestSuite) TestRemoveCurrentActiveRevisionRefused(c *C) {
+func (s *snapmgrTestSuite) TestRemoveLastActiveRevisionRunThrough(c *C) {
+	// run through a removal of a last active revision of a snap where revision
+	// is expliciltly provided in options, making it effectively the same as
+	// regular removal of all revisions of the snap
 	si := snap.SideInfo{
 		RealName: "some-snap",
 		Revision: snap.R(2),
@@ -1058,24 +1071,113 @@ func (s *snapmgrTestSuite) TestRemoveCurrentActiveRevisionRefused(c *C) {
 		SnapType: "app",
 	})
 
-	_, err := snapstate.Remove(s.state, "some-snap", snap.R(2), nil)
+	chg := s.state.NewChange("remove", "remove a snap")
+	// remove last active revision, is essentially like snap remove
+	ts, err := snapstate.Remove(s.state, "some-snap", snap.R(2), nil)
+	c.Assert(err, IsNil)
+	chg.AddAll(ts)
 
-	c.Check(err, ErrorMatches, `cannot remove active revision 2 of snap "some-snap"`)
+	s.settle(c)
+
+	c.Check(len(s.fakeBackend.ops), Equals, 13)
+	expected := fakeOps{
+		{
+			op:    "auto-disconnect:Doing",
+			name:  "some-snap",
+			revno: snap.R(2),
+		},
+		{
+			op:   "remove-snap-aliases",
+			name: "some-snap",
+		},
+		{
+			op:   "unlink-snap",
+			path: filepath.Join(dirs.SnapMountDir, "some-snap/2"),
+		},
+		{
+			op:    "remove-profiles:Doing",
+			name:  "some-snap",
+			revno: snap.R(2),
+		},
+		{
+			op:   "remove-snap-data",
+			path: filepath.Join(dirs.SnapMountDir, "some-snap/2"),
+		},
+		{
+			op:   "remove-snap-common-data",
+			path: filepath.Join(dirs.SnapMountDir, "some-snap/2"),
+		},
+		{
+			op:   "remove-snap-save-data",
+			path: filepath.Join(dirs.SnapDataSaveDir, "some-snap"),
+		},
+		{
+			op:   "remove-snap-data-dir",
+			name: "some-snap",
+			path: filepath.Join(dirs.SnapDataDir, "some-snap"),
+		},
+		{
+			op:    "remove-snap-files",
+			path:  filepath.Join(dirs.SnapMountDir, "some-snap/2"),
+			stype: "app",
+		},
+		{
+			op:   "remove-snap-mount-units",
+			name: "some-snap",
+		},
+		{
+			op:   "discard-namespace",
+			name: "some-snap",
+		},
+		{
+			op:   "remove-inhibit-lock",
+			name: "some-snap",
+		},
+		{
+			op:   "remove-snap-dir",
+			name: "some-snap",
+			path: filepath.Join(dirs.SnapMountDir, "some-snap"),
+		},
+	}
+	// start with an easier-to-read error if this fails:
+	c.Assert(s.fakeBackend.ops.Ops(), DeepEquals, expected.Ops())
+	c.Assert(s.fakeBackend.ops, DeepEquals, expected)
+
+	// verify snapSetup info
+	tasks := ts.Tasks()
+	for _, t := range tasks {
+		c.Logf("task kind: %v", t.Kind())
+		if t.Kind() == "save-snapshot" || t.Kind() == "run-hook" {
+			continue
+		}
+
+		_, err := snapstate.TaskSnapSetup(t)
+		c.Assert(err, IsNil)
+	}
+
+	// verify that the snap was removed and no more snaps remain in the system
+	var snapst snapstate.SnapState
+	err = snapstate.Get(s.state, "some-snap", &snapst)
+	c.Assert(err, testutil.ErrorIs, state.ErrNoState)
 }
 
 func (s *snapmgrTestSuite) TestRemoveCurrentRevisionOfSeveralRefused(c *C) {
-	si := snap.SideInfo{
+	si2 := snap.SideInfo{
 		RealName: "some-snap",
 		Revision: snap.R(2),
 	}
+	si1 := snap.SideInfo{
+		RealName: "some-snap",
+		Revision: snap.R(1),
+	}
 
 	s.state.Lock()
 	defer s.state.Unlock()
 
 	snapstate.Set(s.state, "some-snap", &snapstate.SnapState{
 		Active:   true,
-		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{&si, &si}),
-		Current:  si.Revision,
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{&si2, &si1}),
+		Current:  si2.Revision,
 		SnapType: "app",
 	})
 
@@ -1565,6 +1667,10 @@ func (s *snapmgrTestSuite) TestRemoveManyUndoRestoresCurrent(c *C) {
 			op:   "link-snap",
 			path: filepath.Join(dirs.SnapMountDir, "some-snap/1"),
 		},
+		{
+			op:     "maybe-set-next-boot",
+			isUndo: true,
+		},
 		{
 			op: "update-aliases",
 		},
@@ -1811,7 +1917,7 @@ func (s *snapmgrTestSuite) TestRemoveKeepsGatingDataIfNotLastRevision(c *C) {
 func (s *validationSetsSuite) removeSnapReferencedByValidationSet(c *C, presence string) error {
 	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		vs := snapasserts.NewValidationSets()
-		someSnap := map[string]interface{}{
+		someSnap := map[string]any{
 			"id":       "yOqKhntON3vR7kwEbVPsILm7bUViPDzx",
 			"name":     "some-snap",
 			"presence": presence,
@@ -1868,7 +1974,7 @@ func (s *validationSetsSuite) TestRemoveInvalidSnapOK(c *C) {
 func (s *validationSetsSuite) TestRemoveSnapRequiredByValidationSetAtSpecificRevisionRefused(c *C) {
 	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		vs := snapasserts.NewValidationSets()
-		someSnap := map[string]interface{}{
+		someSnap := map[string]any{
 			"id":       "yOqKhntON3vR7kwEbVPsILm7bUViPDzx",
 			"name":     "some-snap",
 			"presence": "required",
@@ -1921,7 +2027,7 @@ func (s *validationSetsSuite) TestRemoveSnapRequiredByValidationSetAtSpecificRev
 func (s *validationSetsSuite) TestRemoveSnapRequiredByValidationSetAtSpecificRevisionNotActive(c *C) {
 	restore := snapstate.MockEnforcedValidationSets(func(st *state.State, extraVss ...*asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 		vs := snapasserts.NewValidationSets()
-		someSnap := map[string]interface{}{
+		someSnap := map[string]any{
 			"id":       "yOqKhntON3vR7kwEbVPsILm7bUViPDzx",
 			"name":     "some-snap",
 			"presence": "required",
@@ -2059,3 +2165,356 @@ func (s *snapmgrTestSuite) TestRemoveManyWithPurge(c *C) {
 	}
 
 }
+
+func (s *snapmgrTestSuite) TestRemoveWithCompsTasks(c *C) {
+	const snapName = "snap1"
+	const comp1name = "comp1"
+	const comp2name = "comp2"
+
+	cref1 := naming.NewComponentRef(snapName, comp1name)
+	cref2 := naming.NewComponentRef(snapName, comp2name)
+
+	s.AddCleanup(snapstate.MockReadComponentInfo(func(compMntDir string,
+		snapInfo *snap.Info, csi *snap.ComponentSideInfo) (*snap.ComponentInfo, error) {
+		switch csi.Component.ComponentName {
+		case comp1name:
+			return &snap.ComponentInfo{
+				Component:         cref1,
+				Type:              snap.StandardComponent,
+				ComponentSideInfo: *csi,
+			}, nil
+		case comp2name:
+			return &snap.ComponentInfo{
+				Component:         cref2,
+				Type:              snap.StandardComponent,
+				ComponentSideInfo: *csi,
+			}, nil
+		}
+		return nil, errors.New("unexpected component")
+	}))
+
+	s.AddCleanup(snapstate.MockSnapReadInfo(func(name string, si *snap.SideInfo) (*snap.Info, error) {
+		info := &snap.Info{
+			SuggestedName: name,
+			SideInfo:      *si,
+			SnapType:      snap.TypeApp,
+			Components: map[string]*snap.Component{
+				comp1name: {Name: comp1name, Type: snap.StandardComponent},
+				comp2name: {Name: comp2name, Type: snap.StandardComponent},
+			},
+		}
+		info.Apps = map[string]*snap.AppInfo{
+			"app": {Snap: info, Name: "app"},
+		}
+		return info, nil
+	}))
+
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	siRev1 := &snap.SideInfo{RealName: snapName, Revision: snap.R(1)}
+	siRev2 := &snap.SideInfo{RealName: snapName, Revision: snap.R(2)}
+	comp1si := snap.NewComponentSideInfo(cref1, snap.R(11))
+	comp11si := snap.NewComponentSideInfo(cref1, snap.R(111))
+	comp2si := snap.NewComponentSideInfo(cref2, snap.R(22))
+	comp22si := snap.NewComponentSideInfo(cref2, snap.R(222))
+	seq := snapstatetest.NewSequenceFromRevisionSideInfos(
+		[]*sequence.RevisionSideState{
+			sequence.NewRevisionSideState(siRev1,
+				[]*sequence.ComponentState{
+					sequence.NewComponentState(
+						comp1si, snap.StandardComponent),
+					sequence.NewComponentState(
+						comp2si, snap.StandardComponent),
+				}),
+			sequence.NewRevisionSideState(siRev2,
+				[]*sequence.ComponentState{
+					sequence.NewComponentState(
+						comp11si, snap.StandardComponent),
+					sequence.NewComponentState(
+						comp22si, snap.StandardComponent),
+				}),
+		})
+	snapstate.Set(s.state, snapName, &snapstate.SnapState{
+		Active:   true,
+		Sequence: seq,
+		Current:  snap.R(1),
+		SnapType: "app",
+	})
+
+	ts, err := snapstate.Remove(s.state, snapName, snap.R(0), nil)
+	c.Assert(err, IsNil)
+
+	c.Assert(s.state.TaskCount(), Equals, len(ts.Tasks()))
+	c.Assert(taskKinds(ts.Tasks()), DeepEquals, []string{
+		"stop-snap-services",
+		"run-hook[remove]", // component remove hook
+		"run-hook[remove]", // component remove hook
+		"run-hook[remove]", // snap remove hook
+		"auto-disconnect",
+		"save-snapshot",
+		"remove-aliases",
+		"unlink-snap",
+		"remove-profiles",
+		"clear-snap",
+		"unlink-component",
+		"discard-component",
+		"unlink-component",
+		"discard-component",
+		"discard-snap",
+		"clear-snap",
+		"unlink-component",
+		"discard-component",
+		"unlink-component",
+		"discard-component",
+		"discard-snap",
+	})
+	verifyStopReason(c, ts, "remove")
+
+	// Run the created tasks
+	chg := s.state.NewChange("remove", "remove a snap")
+	c.Assert(err, IsNil)
+	chg.AddAll(ts)
+
+	s.settle(c)
+
+	expected := fakeOps{
+		{
+			op:    "auto-disconnect:Doing",
+			name:  "snap1",
+			revno: snap.R(1),
+		},
+		{
+			op:   "remove-snap-aliases",
+			name: "snap1",
+		},
+		{
+			op:   "unlink-snap",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/1"),
+		},
+		{
+			op:    "remove-profiles:Doing",
+			name:  "snap1",
+			revno: snap.R(1),
+		},
+		{
+			op:   "remove-snap-data",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/2"),
+		},
+		{
+			op:   "unlink-component",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/components/mnt/comp1/111"),
+		},
+		{
+			op:                "undo-setup-component",
+			containerName:     "snap1+comp1",
+			containerFileName: "snap1+comp1_111.comp",
+		},
+		{
+			op:                "remove-component-dir",
+			containerName:     "snap1+comp1",
+			containerFileName: "snap1+comp1_111.comp",
+		},
+		{
+			op:   "unlink-component",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/components/mnt/comp2/222"),
+		},
+		{
+			op:                "undo-setup-component",
+			containerName:     "snap1+comp2",
+			containerFileName: "snap1+comp2_222.comp",
+		},
+		{
+			op:                "remove-component-dir",
+			containerName:     "snap1+comp2",
+			containerFileName: "snap1+comp2_222.comp",
+		},
+		{
+			op:    "remove-snap-files",
+			path:  filepath.Join(dirs.SnapMountDir, "snap1/2"),
+			stype: "app",
+		},
+		{
+			op:   "remove-snap-data",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/1"),
+		},
+		{
+			op:   "remove-snap-common-data",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/1"),
+		},
+		{
+			op:   "remove-snap-save-data",
+			path: filepath.Join(dirs.SnapDataSaveDir, "snap1"),
+		},
+		{
+			op:   "remove-snap-data-dir",
+			name: "snap1",
+			path: filepath.Join(dirs.SnapDataDir, "snap1"),
+		},
+		{
+			op:   "unlink-component",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/components/mnt/comp1/11"),
+		},
+		{
+			op:                "undo-setup-component",
+			containerName:     "snap1+comp1",
+			containerFileName: "snap1+comp1_11.comp",
+		},
+		{
+			containerName:     "snap1+comp1",
+			containerFileName: "snap1+comp1_11.comp",
+			op:                "remove-component-dir",
+		},
+		{
+			op:   "unlink-component",
+			path: filepath.Join(dirs.SnapMountDir, "snap1/components/mnt/comp2/22"),
+		},
+		{
+			op:                "undo-setup-component",
+			containerName:     "snap1+comp2",
+			containerFileName: "snap1+comp2_22.comp",
+		},
+		{
+			op:                "remove-component-dir",
+			containerName:     "snap1+comp2",
+			containerFileName: "snap1+comp2_22.comp",
+		},
+		{
+			op:    "remove-snap-files",
+			path:  filepath.Join(dirs.SnapMountDir, "snap1/1"),
+			stype: "app",
+		},
+		{
+			op:   "remove-snap-mount-units",
+			name: "snap1",
+		},
+		{
+			op:   "discard-namespace",
+			name: "snap1",
+		},
+		{
+			op:   "remove-inhibit-lock",
+			name: "snap1",
+		},
+		{
+			op:   "remove-snap-dir",
+			name: "snap1",
+			path: filepath.Join(dirs.SnapMountDir, "snap1"),
+		},
+	}
+	// start with an easier-to-read error if this fails:
+	c.Check(len(s.fakeBackend.ops), Equals, len(expected))
+	c.Check(s.fakeBackend.ops[0:5], DeepEquals, expected[0:5])
+	c.Check(s.fakeBackend.ops[11:16], DeepEquals, expected[11:16])
+	c.Check(s.fakeBackend.ops[22:], DeepEquals, expected[22:])
+	// Check component tasks, that can run in parallel so the order is not
+	// deterministic, but still needs to follow an order per component.
+	checkComps := func(ops, exp1, exp2 fakeOps) {
+		var op1idx, op2idx int
+		for _, op := range ops {
+			switch {
+			case op1idx < 3 && reflect.DeepEqual(exp1[op1idx], op):
+				op1idx++
+			case op2idx < 3 && reflect.DeepEqual(exp2[op2idx], op):
+				op2idx++
+			default:
+				c.Error("expected op not found", op)
+			}
+		}
+	}
+	checkComps(s.fakeBackend.ops[5:11], expected[5:8], expected[8:11])
+	checkComps(s.fakeBackend.ops[16:22], expected[16:19], expected[19:22])
+}
+
+func (s *snapmgrTestSuite) TestRemoveWithTerminate(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	snapstate.Set(s.state, "foo", &snapstate.SnapState{
+		Active: true,
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
+			{RealName: "foo", Revision: snap.R(11)},
+		}),
+		Current:  snap.R(11),
+		SnapType: "app",
+	})
+
+	ts, err := snapstate.Remove(s.state, "foo", snap.R(0), &snapstate.RemoveFlags{Terminate: true})
+	c.Assert(err, IsNil)
+
+	c.Assert(taskKinds(ts.Tasks()), DeepEquals, []string{
+		"stop-snap-services",
+		"run-hook[remove]",
+		"auto-disconnect",
+		"kill-snap-apps",
+		"save-snapshot",
+		"remove-aliases",
+		"unlink-snap",
+		"remove-profiles",
+		"clear-snap",
+		"discard-snap",
+	})
+}
+
+func (s *snapmgrTestSuite) TestRemoveWithTerminateAndRevisionSet(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	snapstate.Set(s.state, "foo", &snapstate.SnapState{
+		Active: true,
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
+			{RealName: "foo", Revision: snap.R(11)},
+			{RealName: "foo", Revision: snap.R(10)},
+		}),
+		Current:  snap.R(11),
+		SnapType: "app",
+	})
+
+	_, err := snapstate.Remove(s.state, "foo", snap.R(10), &snapstate.RemoveFlags{Terminate: true})
+	c.Assert(err, ErrorMatches, "cannot terminate running apps unless all revisions are removed")
+}
+
+func (s *snapmgrTestSuite) TestRemoveManyWithTerminate(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	snapstate.Set(s.state, "some-snap", &snapstate.SnapState{
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
+			{RealName: "some-snap", SnapID: "some-snap-id", Revision: snap.R(1)},
+		}),
+		Current:  snap.R(1),
+		SnapType: "app",
+		Active:   true,
+	})
+
+	snapstate.Set(s.state, "foo", &snapstate.SnapState{
+		Sequence: snapstatetest.NewSequenceFromSnapSideInfos([]*snap.SideInfo{
+			{RealName: "foo", SnapID: "foo-id", Revision: snap.R(11)},
+		}),
+		Current:  snap.R(11),
+		SnapType: "app",
+		Active:   true,
+	})
+
+	removed, tss, err := snapstate.RemoveMany(s.state, []string{"some-snap", "foo"}, &snapstate.RemoveFlags{Terminate: true})
+	c.Check(removed, DeepEquals, []string{"some-snap", "foo"})
+	c.Check(tss, NotNil)
+	c.Check(err, IsNil)
+
+	// tasks to take snapshot aren't generated
+	for _, ts := range tss {
+		c.Assert(taskKinds(ts.Tasks()), DeepEquals, []string{
+			"stop-snap-services",
+			"run-hook[remove]",
+			"auto-disconnect",
+			"kill-snap-apps",
+			"save-snapshot",
+			"remove-aliases",
+			"unlink-snap",
+			"remove-profiles",
+			"clear-snap",
+			"discard-snap",
+		})
+	}
+
+}
