@@ -34,6 +34,7 @@ import (
 	sb_plainkey "github.com/snapcore/secboot/plainkey"
 	"golang.org/x/xerrors"
 
+	"github.com/snapcore/snapd/gadget/device"
 	"github.com/snapcore/snapd/kernel/fde"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil/disks"
@@ -54,11 +55,17 @@ var (
 	sbNewLUKS2KeyDataReader         = sbNewLUKS2KeyDataReaderImpl
 	sbSetProtectorKeys              = sb_plainkey.SetProtectorKeys
 	sbGetPrimaryKeyFromKernel       = sb.GetPrimaryKeyFromKernel
+	sbTestLUKS2ContainerKey         = sb.TestLUKS2ContainerKey
+	sbCheckPassphraseEntropy        = sb.CheckPassphraseEntropy
 	disksDevlinks                   = disks.Devlinks
+
+	sbKeyDataChangePassphrase = (*sb.KeyData).ChangePassphrase
 )
 
 func init() {
 	WithSecbootSupport = true
+
+	device.EntropyBits = EntropyBits
 }
 
 type DiskUnlockKey sb.DiskUnlockKey
@@ -169,11 +176,7 @@ func UnlockVolumeUsingSealedKeyIfEncrypted(disk disks.Disk, name string, sealedE
 
 	const allowPassphrase = true
 	options := activateVolOpts(opts.AllowRecoveryKey, allowPassphrase, partDevice)
-	authRequestor, err := newAuthRequestor()
-	if err != nil {
-		res.UnlockMethod = NotUnlocked
-		return res, fmt.Errorf("internal error: cannot build an auth requestor: %v", err)
-	}
+	authRequestor := newAuthRequestor()
 
 	// Non-nil FDEHookKeyV1 indicates that V1 hook key is used
 	if loadedKey.FDEHookKeyV1 != nil {
@@ -335,8 +338,7 @@ func DeactivateVolume(volumeName string) error {
 // temporary and is expected to be used with a BootstrappedContainer,
 // and removed by calling RemoveBootstrapKey.
 func AddBootstrapKeyOnExistingDisk(node string, newKey keys.EncryptionKey) error {
-	const defaultPrefix = "ubuntu-fde"
-	unlockKey, err := sbGetDiskUnlockKeyFromKernel(defaultPrefix, node, false)
+	unlockKey, err := sbGetDiskUnlockKeyFromKernel(defaultKeyringPrefix, node, false)
 	if err != nil {
 		return fmt.Errorf("cannot get key for unlocked disk %s: %v", node, err)
 	}
@@ -354,7 +356,7 @@ func AddBootstrapKeyOnExistingDisk(node string, newKey keys.EncryptionKey) error
 // WARNING: this function is not always atomic. If cryptsetup is too
 // old, it will try to copy and delete keys instead. Please avoid
 // using this function in new code.
-func RenameKeys(node string, renames map[string]string) error {
+func RenameKeysForFactoryReset(node string, renames map[string]string) error {
 	targets := make(map[string]bool)
 
 	for _, renameTo := range renames {
@@ -367,6 +369,7 @@ func RenameKeys(node string, renames map[string]string) error {
 
 	// TODO:FDEM:FIX: listing keys, then modifying could be a TOCTOU issue.
 	// we expect here nothing else is messing with the key slots.
+	// XXX: include recovery key slots sbListLUKS2ContainerRecoveryKeyNames
 	slots, err := sbListLUKS2ContainerUnlockKeyNames(node)
 	if err != nil {
 		return fmt.Errorf("cannot list slots in partition save partition: %v", err)
@@ -397,9 +400,18 @@ func RenameKeys(node string, renames map[string]string) error {
 	return nil
 }
 
+// RenameContainerKey renames a key slot on LUKS2 container. An error
+// is returned if cryptsetup does not support --token-replace option.
+func RenameContainerKey(devicePath, oldName, newName string) error {
+	return sbRenameLUKS2ContainerKey(devicePath, oldName, newName)
+}
+
 // DeleteKeys delete key slots on a LUKS2 container. Slots that do not
 // exist are ignored.
+//
+// XXX: s/DeleteKey/DeleteContainerKey
 func DeleteKeys(node string, matches map[string]bool) error {
+	// XXX: include recovery key slots sbListLUKS2ContainerRecoveryKeyNames
 	slots, err := sbListLUKS2ContainerUnlockKeyNames(node)
 	if err != nil {
 		return fmt.Errorf("cannot list slots in partition save partition: %v", err)
@@ -416,6 +428,11 @@ func DeleteKeys(node string, matches map[string]bool) error {
 	return nil
 }
 
+// DeleteContainerKey deletes a key slot on a LUKS2 container.
+func DeleteContainerKey(devicePath, slotName string) error {
+	return sbDeleteLUKS2ContainerKey(devicePath, slotName)
+}
+
 func findPrimaryKey(devicePath string) ([]byte, error) {
 	const remove = false
 	p, err := sbGetPrimaryKeyFromKernel(keyringPrefix, devicePath, remove)
@@ -529,3 +546,126 @@ func sbCopyAndRemoveLUKS2ContainerKeyImpl(devicePath, keyslotName, renameTo stri
 }
 
 var sbCopyAndRemoveLUKS2ContainerKey = sbCopyAndRemoveLUKS2ContainerKeyImpl
+
+// GetPrimaryKey finds the primary from the keyring based on the path of
+// encrypted devices. If it does not find any primary in the keyring,
+// it then tries to read the key from a fallback key file.
+func GetPrimaryKey(devices []string, fallbackKeyFile string) ([]byte, error) {
+	for _, device := range devices {
+		primaryKey, err := findPrimaryKey(device)
+		if err == nil {
+			return primaryKey, nil
+		}
+		if !errors.Is(err, sb.ErrKernelKeyNotFound) {
+			return nil, err
+		}
+	}
+
+	primaryKey, err := os.ReadFile(fallbackKeyFile)
+	if err != nil {
+		return nil, fmt.Errorf("could not find primary in keyring and cannot read fallback primary key file %s: %w", fallbackKeyFile, err)
+	}
+	return primaryKey, nil
+}
+
+// CheckRecoveryKey tests that the specified recovery key unlocks the
+// device at the specified path.
+func CheckRecoveryKey(devicePath string, rkey keys.RecoveryKey) error {
+	if !sbTestLUKS2ContainerKey(devicePath, rkey[:]) {
+		return fmt.Errorf("invalid recovery key for %s", devicePath)
+	}
+	return nil
+}
+
+// ListContainerRecoveryKeyNames lists the names of key slots on the specified
+// device configured as recovery slots.
+//
+// Note: This only supports LUKS2 containers.
+func ListContainerRecoveryKeyNames(devicePath string) ([]string, error) {
+	return sbListLUKS2ContainerRecoveryKeyNames(devicePath)
+}
+
+// ListContainerUnlockKeyNames lists the names of key slots on the specified
+// device configured as normal unlock slots (the keys associated with these
+// should be protected by the platform's secure device).
+//
+// Note: This only supports LUKS2 containers.
+func ListContainerUnlockKeyNames(devicePath string) ([]string, error) {
+	return sbListLUKS2ContainerUnlockKeyNames(devicePath)
+}
+
+type keyData struct {
+	kd *sb.KeyData
+}
+
+// AuthMode indicates the authentication mechanisms enabled for this key data.
+func (k *keyData) AuthMode() device.AuthMode {
+	switch k.kd.AuthMode() {
+	case sb.AuthModeNone:
+		return device.AuthModeNone
+	case sb.AuthModePassphrase:
+		return device.AuthModePassphrase
+	// TODO:FDEM: add AuthModePIN when it lands in secboot
+	default:
+		return ""
+	}
+}
+
+// PlatformName returns the name of the platform that handles this key data.
+func (k *keyData) PlatformName() string {
+	return k.kd.PlatformName()
+}
+
+// Role indicates the role of this key.
+func (k *keyData) Roles() []string {
+	if k.kd.Role() == "" {
+		return nil
+	}
+	return []string{k.kd.Role()}
+}
+
+func (k *keyData) ChangePassphrase(oldPassphrase, newPassphrase string) error {
+	return sbKeyDataChangePassphrase(k.kd, oldPassphrase, newPassphrase)
+}
+
+func (k *keyData) WriteTokenAtomic(devicePath, slotName string) error {
+	writer, err := newLUKS2KeyDataWriter(devicePath, slotName)
+	if err != nil {
+		return err
+	}
+	return k.kd.WriteAtomic(writer)
+}
+
+// ReadContainerKeyData reads key slot key data for the specified device and slot name.
+//
+// Note: This only supports key datas stored in LUKS2 tokens.
+func ReadContainerKeyData(devicePath, slotName string) (KeyData, error) {
+	kd, err := readKeyToken(devicePath, slotName)
+	if err != nil {
+		return nil, err
+	}
+
+	return &keyData{kd: kd}, nil
+}
+
+// EntropyBits calculates entropy for PINs and passphrases.
+//
+// PINs will be supplied as a numeric passphrase.
+func EntropyBits(passphrase string) (uint32, error) {
+	stats, err := sbCheckPassphraseEntropy(passphrase)
+	if err != nil {
+		return 0, err
+	}
+	return stats.EntropyBits, nil
+}
+
+// AddContainerRecoveryKey adds a new recovery key to specified device.
+//
+// Note: The unlock key is implicitly obtained from the kernel keyring.
+func AddContainerRecoveryKey(devicePath string, slotName string, rkey keys.RecoveryKey) error {
+	unlockKey, err := sbGetDiskUnlockKeyFromKernel(defaultKeyringPrefix, devicePath, false)
+	if err != nil {
+		return fmt.Errorf("cannot get key from kernel keyring for unlocked disk %s: %v", devicePath, err)
+	}
+	return sbAddLUKS2ContainerRecoveryKey(devicePath, slotName, unlockKey, sb.RecoveryKey(rkey))
+}
