@@ -32,6 +32,7 @@ import (
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/channel"
 	"github.com/snapcore/snapd/snap/naming"
+	"github.com/snapcore/snapd/snap/squashfs"
 	"github.com/snapcore/snapd/strutil"
 )
 
@@ -55,6 +56,13 @@ type Options struct {
 	// ManifestPath if set, specifies the file path where the
 	// seed.manifest file should be written.
 	ManifestPath string
+
+	// IgnoreOptionFileExtentions if set, snaps and components will not be
+	// required to end in .snap or .comp, respectively.
+	IgnoreOptionFileExtentions bool
+
+	// Assertions to inject into the built image
+	ExtraAssertions []asserts.Assertion
 }
 
 // manifest returns either the manifest already provided by the
@@ -66,15 +74,24 @@ func (opts *Options) manifest() *Manifest {
 	return opts.Manifest
 }
 
-// OptionsSnap represents an options-referred snap with its option values.
-// E.g. a snap passed to ubuntu-image via --snap.
-// If Name is set the snap is from the store. If Path is set the snap
-// is local at Path location.
+// OptionsComponent represents an options-referred snap with its option values.
+// E.g. a component passed to ubuntu-image via --comp <snap_name>+<comp_name>.
+type OptionsComponent struct {
+	Name string
+	Path string
+}
+
+// OptionsSnap represents an options-referred snap with its option values. E.g.
+// a snap passed to ubuntu-image via --snap. If Name is set the snap is from
+// the store. If Path is set the snap is local at Path location. Components are
+// the components passed via the --comp option. If there is a component option
+// but no matching snap option, an implicit OptionsSnap is created.
 type OptionsSnap struct {
-	Name    string
-	SnapID  string
-	Path    string
-	Channel string
+	Name       string
+	SnapID     string
+	Path       string
+	Channel    string
+	Components []OptionsComponent
 }
 
 func (s *OptionsSnap) SnapName() string {
@@ -85,6 +102,19 @@ func (s *OptionsSnap) ID() string {
 	return s.SnapID
 }
 
+func (s *OptionsSnap) Component(compName string) *OptionsComponent {
+	for _, optComp := range s.Components {
+		if optComp.Name == compName {
+			return &optComp
+		}
+	}
+	return nil
+}
+
+func (s *OptionsSnap) HasComponent(compName string) bool {
+	return s.Component(compName) != nil
+}
+
 var _ naming.SnapRef = (*OptionsSnap)(nil)
 
 // SeedSnap holds details of a snap being added to a seed.
@@ -93,6 +123,11 @@ type SeedSnap struct {
 	Channel string
 	Path    string
 
+	// Components are the components of the snap to be copied to the seed.
+	// If using local components, the slice will be set by
+	// Writer.AddComponentsToSnap(), as we don't know initially which ones
+	// are being included in this way.
+	Components []SeedComponent
 	// Info is the *snap.Info for the seed snap, filling this is
 	// delegated to the Writer using code, via Writer.SetInfo.
 	Info *snap.Info
@@ -107,6 +142,14 @@ type SeedSnap struct {
 	optionSnap *OptionsSnap
 }
 
+// SeedComponent holds details of a component being added to a seed.
+type SeedComponent struct {
+	naming.ComponentRef
+	Path string
+
+	Info *snap.ComponentInfo
+}
+
 func (sn *SeedSnap) modes() []string {
 	if sn.modelSnap == nil {
 		// run is the assumed mode for extra snaps not listed
@@ -186,6 +229,7 @@ type Writer struct {
 	expectedStep writerStep
 
 	modelRefs []*asserts.Ref
+	extraRefs []*asserts.Ref
 
 	optionsSnaps []*OptionsSnap
 	// consumedOptSnapNum counts which options snaps have been consumed
@@ -252,10 +296,12 @@ type tree interface {
 	mkFixedDirs() error
 
 	snapPath(*SeedSnap) (string, error)
+	componentPath(*SeedSnap, *SeedComponent) (string, error)
 
 	localSnapPath(*SeedSnap) (string, error)
+	localComponentPath(*SeedComponent, string) (string, error)
 
-	writeAssertions(db asserts.RODatabase, modelRefs []*asserts.Ref, snapsFromModel []*SeedSnap, extraSnaps []*SeedSnap) error
+	writeAssertions(db asserts.RODatabase, modelRefs []*asserts.Ref, extraRefs []*asserts.Ref, snapsFromModel []*SeedSnap, extraSnaps []*SeedSnap) error
 
 	writeMeta(snapsFromModel []*SeedSnap, extraSnaps []*SeedSnap) error
 }
@@ -376,10 +422,30 @@ func (w *Writer) checkStep(thisStep writerStep) error {
 }
 
 // warningf adds a warning that can be later retrieved via Warnings.
-func (w *Writer) warningf(format string, a ...interface{}) {
+func (w *Writer) warningf(format string, a ...any) {
 	w.warnings = append(w.warnings, fmt.Sprintf(format, a...))
 }
 
+func (w *Writer) validateComponent(optComp *OptionsComponent) error {
+	if optComp.Name != "" {
+		if optComp.Path != "" {
+			return fmt.Errorf("cannot specify both name and path for component %q",
+				optComp.Name)
+		}
+		if err := snap.ValidateName(optComp.Name); err != nil {
+			return err
+		}
+	} else {
+		if !strings.HasSuffix(optComp.Path, ".comp") && !w.opts.IgnoreOptionFileExtentions {
+			return fmt.Errorf("local option component %q does not end in .comp", optComp.Path)
+		}
+		if !osutil.FileExists(optComp.Path) {
+			return fmt.Errorf("local option component %q does not exist", optComp.Path)
+		}
+	}
+	return nil
+}
+
 // SetOptionsSnaps accepts options-referred snaps represented as OptionsSnap.
 func (w *Writer) SetOptionsSnaps(optSnaps []*OptionsSnap) error {
 	if err := w.checkStep(setOptionsSnapsStep); err != nil {
@@ -412,7 +478,7 @@ func (w *Writer) SetOptionsSnaps(optSnaps []*OptionsSnap) error {
 			}
 			w.byNameOptSnaps.Add(sn)
 		} else {
-			if !strings.HasSuffix(sn.Path, ".snap") {
+			if !strings.HasSuffix(sn.Path, ".snap") && !w.opts.IgnoreOptionFileExtentions {
 				return fmt.Errorf("local option snap %q does not end in .snap", sn.Path)
 			}
 			if !osutil.FileExists(sn.Path) {
@@ -431,6 +497,11 @@ func (w *Writer) SetOptionsSnaps(optSnaps []*OptionsSnap) error {
 				return err
 			}
 		}
+		for _, comp := range sn.Components {
+			if err := w.validateComponent(&comp); err != nil {
+				return err
+			}
+		}
 		if local {
 			if w.localSnaps == nil {
 				w.localSnaps = make(map[*OptionsSnap]*SeedSnap)
@@ -566,6 +637,22 @@ func (w *Writer) Start(db asserts.RODatabase, f SeedAssertionFetcher) error {
 
 	w.modelRefs = f.Refs()
 
+	if len(w.opts.ExtraAssertions) != 0 {
+
+		f.AddExtraAssertions(w.opts.ExtraAssertions)
+
+		for _, extraAssertion := range w.opts.ExtraAssertions {
+			if err := f.Save(extraAssertion); err != nil {
+				return fmt.Errorf(
+					"cannot fetch and check prerequisites for an injected assertion: %v",
+					err,
+				)
+			}
+		}
+
+		w.extraRefs = f.Refs()[len(w.modelRefs):]
+	}
+
 	if err := w.tree.mkFixedDirs(); err != nil {
 		return err
 	}
@@ -653,9 +740,11 @@ func (w *Writer) InfoDerived() error {
 	return nil
 }
 
-// SetInfo sets Info of the SeedSnap and possibly computes its
-// destination Path.
-func (w *Writer) SetInfo(sn *SeedSnap, info *snap.Info) error {
+// SetInfo sets info and seedComps (which is a map of component names
+// to SeedComponent) in the SeedSnap sn and computes destination paths
+// for all if coming from the store. If the components do not come
+// from the store, some additional checks are performed.
+func (w *Writer) SetInfo(sn *SeedSnap, info *snap.Info, seedComps map[string]*SeedComponent) error {
 	if info.NeedsDevMode() {
 		if err := w.policy.allowsDangerousFeatures(); err != nil {
 			return err
@@ -665,8 +754,22 @@ func (w *Writer) SetInfo(sn *SeedSnap, info *snap.Info) error {
 
 	if sn.local {
 		sn.SnapRef = info
-		// nothing more to do
-		return nil
+		return w.assignLocalComponents(sn, seedComps)
+	}
+
+	for i := range sn.Components {
+		seedComp, ok := seedComps[sn.Components[i].ComponentName]
+		if !ok {
+			return fmt.Errorf("store did not return information about %s",
+				sn.Components[i].ComponentName)
+		}
+		sn.Components[i] = *seedComp
+		// Fill the path as this is a non-local component
+		compPath, err := w.tree.componentPath(sn, &sn.Components[i])
+		if err != nil {
+			return err
+		}
+		sn.Components[i].Path = compPath
 	}
 
 	p, err := w.tree.snapPath(sn)
@@ -674,6 +777,38 @@ func (w *Writer) SetInfo(sn *SeedSnap, info *snap.Info) error {
 		return err
 	}
 	sn.Path = p
+
+	return nil
+}
+
+type byCompName []SeedComponent
+
+func (c byCompName) Len() int           { return len(c) }
+func (c byCompName) Less(i, j int) bool { return c[i].ComponentName < c[j].ComponentName }
+func (c byCompName) Swap(i, j int)      { c[i], c[j] = c[j], c[i] }
+
+func (w *Writer) assignLocalComponents(sn *SeedSnap, seedComps map[string]*SeedComponent) error {
+	for _, seedComp := range seedComps {
+		// Check if the component is defined by the snap
+		compInSnap, ok := sn.Info.Components[seedComp.ComponentName]
+		if !ok {
+			return fmt.Errorf("component %s is not defined by snap %s",
+				seedComp.ComponentName, sn.SnapName())
+		}
+		// and if types match
+		if compInSnap.Type != seedComp.Info.Type {
+			return fmt.Errorf("component %s has type %s while snap %s defines type %s for it",
+				seedComp.ComponentName, seedComp.Info.Type,
+				sn.SnapName(), compInSnap.Type)
+		}
+
+		// now we can add to the snap
+		sn.Components = append(sn.Components, *seedComp)
+	}
+
+	// Sort for deterministic download order and tests
+	sort.Sort(byCompName(sn.Components))
+
 	return nil
 }
 
@@ -727,11 +862,41 @@ func (w *Writer) modelSnapToSeed(modSnap *asserts.ModelSnap) (*SeedSnap, error)
 			// by an OptionsSnap entry is skipped
 			return nil, errSkipOptional
 		}
+		seedCompsMap := make(map[string]SeedComponent, len(modSnap.Components))
+		for comp, modComp := range modSnap.Components {
+			// optional snap not confirmed (no options or not in options), skipping
+			if modComp.Presence == "optional" &&
+				(optSnap == nil || !optSnap.HasComponent(comp)) {
+				continue
+			}
+			seedCompsMap[comp] = SeedComponent{
+				ComponentRef: naming.NewComponentRef(modSnap.Name, comp),
+			}
+		}
+		// We add also components in command options if the model allows it
+		if optSnap != nil {
+			for _, comp := range optSnap.Components {
+				if _, ok := seedCompsMap[comp.Name]; ok {
+					continue
+				}
+				if err := w.policy.allowsDangerousFeatures(); err != nil {
+					return nil, err
+				}
+				seedCompsMap[comp.Name] = SeedComponent{
+					ComponentRef: naming.NewComponentRef(modSnap.Name, comp.Name),
+				}
+			}
+		}
+		seedComps := make([]SeedComponent, 0, len(seedCompsMap))
+		for _, sc := range seedCompsMap {
+			seedComps = append(seedComps, sc)
+		}
 		sn = &SeedSnap{
 			SnapRef: modSnap,
 
 			local:      false,
 			optionSnap: optSnap,
+			Components: seedComps,
 		}
 	} else {
 		optSnap = sn.optionSnap
@@ -826,11 +991,18 @@ func (w *Writer) extraSnapToSeed(optSnap *OptionsSnap) (*SeedSnap, error) {
 	sn := w.localSnaps[optSnap]
 	if sn == nil {
 		// not local, to download
+		seedComps := make([]SeedComponent, 0, len(optSnap.Components))
+		for _, optComp := range optSnap.Components {
+			seedComps = append(seedComps, SeedComponent{
+				ComponentRef: naming.NewComponentRef(optSnap.Name, optComp.Name),
+			})
+		}
 		sn = &SeedSnap{
 			SnapRef: optSnap,
 
 			local:      false,
 			optionSnap: optSnap,
+			Components: seedComps,
 		}
 	}
 	if sn.SnapName() == "" {
@@ -948,7 +1120,8 @@ func (w *Writer) resolveChannel(whichSnap string, modSnap *asserts.ModelSnap, op
 	return resChannel, nil
 }
 
-func (w *Writer) checkBase(info *snap.Info, modes []string) error {
+func (w *Writer) checkBase(sn *SeedSnap) error {
+	info := sn.Info
 	// Validity check, note that we could support this case
 	// if we have a use-case but it requires changes in the
 	// devicestate/firstboot.go ordering code.
@@ -961,7 +1134,16 @@ func (w *Writer) checkBase(info *snap.Info, modes []string) error {
 		return nil
 	}
 
-	return w.policy.checkBase(info, modes, w.availableByMode)
+	modes := sn.modes()
+	err := w.policy.checkBase(info, modes, w.availableByMode)
+	if err != nil {
+		// in dangerous mode check base only at the end
+		// allowing overrides to provide the new base as well
+		if w.policy.allowsDangerousFeatures() == nil && sn.optionSnap != nil {
+			return nil
+		}
+	}
+	return err
 }
 
 func (w *Writer) recordUsageWithThePolicy(modSnaps []*asserts.ModelSnap) {
@@ -1143,9 +1325,7 @@ func (w *Writer) downloaded(seedSnaps []*SeedSnap, fetchAsserts AssertsFetchFunc
 			}
 		}
 
-		modes := sn.modes()
-
-		if err := w.checkBase(info, modes); err != nil {
+		if err := w.checkBase(sn); err != nil {
 			return err
 		}
 	}
@@ -1387,9 +1567,11 @@ func (w *Writer) validationSets() (*snapasserts.ValidationSets, error) {
 
 func (w *Writer) installedSnaps() []*snapasserts.InstalledSnap {
 	installedSnap := func(snap *SeedSnap) *snapasserts.InstalledSnap {
-		return snapasserts.NewInstalledSnap(snap.SnapName(), snap.ID(), snap.Info.Revision)
+		return snapasserts.NewInstalledSnap(snap.SnapName(), snap.ID(), snap.Info.Revision, nil)
 	}
 
+	// TODO:COMPS: add components
+
 	var installedSnaps []*snapasserts.InstalledSnap
 	for _, sn := range w.snapsFromModel {
 		installedSnaps = append(installedSnaps, installedSnap(sn))
@@ -1457,12 +1639,17 @@ func (w *Writer) SeedSnaps(copySnap func(name, src, dst string) error) error {
 				}
 			} else {
 				var snapPath func(*SeedSnap) (string, error)
+				var compPath func(*SeedComponent, string) (string, error)
 				if sn.Info.ID() != "" {
 					// actually asserted
 					snapPath = w.tree.snapPath
+					compPath = func(sc *SeedComponent, snapVersion string) (string, error) {
+						return w.tree.componentPath(sn, sc)
+					}
 				} else {
 					// purely local
 					snapPath = w.tree.localSnapPath
+					compPath = w.tree.localComponentPath
 				}
 				dst, err := snapPath(sn)
 				if err != nil {
@@ -1471,7 +1658,19 @@ func (w *Writer) SeedSnaps(copySnap func(name, src, dst string) error) error {
 				if err := copySnap(info.SnapName(), sn.Path, dst); err != nil {
 					return err
 				}
-				// record final destination path
+				// copy components
+				for i, comp := range sn.Components {
+					compDst, err := compPath(&comp, sn.Info.Version)
+					if err != nil {
+						return err
+					}
+					if err := copySnap(comp.ComponentRef.String(), comp.Path, compDst); err != nil {
+						return err
+					}
+					// record final destination path (for correct options.yaml)
+					sn.Components[i].Path = compDst
+				}
+				// record final destination path (for correct options.yaml)
 				sn.Path = dst
 			}
 			if !info.Revision.Unset() {
@@ -1526,7 +1725,7 @@ func (w *Writer) WriteMeta() error {
 	snapsFromModel := w.snapsFromModel
 	extraSnaps := w.extraSnaps
 
-	if err := w.tree.writeAssertions(w.db, w.modelRefs, snapsFromModel, extraSnaps); err != nil {
+	if err := w.tree.writeAssertions(w.db, w.modelRefs, w.extraRefs, snapsFromModel, extraSnaps); err != nil {
 		return err
 	}
 
@@ -1587,3 +1786,51 @@ func (w *Writer) UnassertedSnaps() ([]naming.SnapRef, error) {
 	}
 	return res, nil
 }
+
+func (w *Writer) VerifySnapBootstrapCompatibility() error {
+	var kernelSnap, snapdSnap *SeedSnap
+
+	saveSnap := func(sn *SeedSnap) {
+		switch sn.Info.SnapType {
+		case snap.TypeSnapd:
+			snapdSnap = sn
+		case snap.TypeKernel:
+			kernelSnap = sn
+		}
+	}
+	for _, sn := range w.snapsFromModel {
+		saveSnap(sn)
+	}
+	for _, sn := range w.extraSnaps {
+		saveSnap(sn)
+	}
+
+	if kernelSnap == nil || snapdSnap == nil {
+		return nil
+	}
+
+	kernelVersion, _, err := snap.SnapdInfoFromSnapFile(squashfs.New(kernelSnap.Path), snap.TypeKernel)
+	if err != nil {
+		return fmt.Errorf("error while reading snapd-info from kernel snap: %w", err)
+	}
+	snapdVersion, _, err := snap.SnapdInfoFromSnapFile(squashfs.New(snapdSnap.Path), snap.TypeSnapd)
+	if err != nil {
+		return fmt.Errorf("error while reading snapd-info from snapd snap: %w", err)
+	}
+
+	res, err := strutil.VersionCompare(snapdVersion, "2.68")
+	if err != nil {
+		return fmt.Errorf("could not parse version %s: %w", snapdVersion, err)
+	}
+	if res >= 0 {
+		res, err = strutil.VersionCompare(kernelVersion, "2.68")
+		if err != nil {
+			return fmt.Errorf("could not parse version %s: %w", kernelVersion, err)
+		}
+		if res < 0 {
+			return fmt.Errorf("snapd 2.68+ is not compatible with a kernel containing snapd prior to 2.68")
+		}
+	}
+
+	return nil
+}
