@@ -23,6 +23,7 @@ import (
 	"context"
 	"encoding/base64"
 	"fmt"
+	"math/rand"
 	"net/http"
 	"net/url"
 	"os"
@@ -42,6 +43,7 @@ import (
 	"github.com/snapcore/snapd/progress"
 	"github.com/snapcore/snapd/seed/seedtest"
 	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/naming"
 	"github.com/snapcore/snapd/store"
 	"github.com/snapcore/snapd/store/tooling"
 	"github.com/snapcore/snapd/testutil"
@@ -85,15 +87,18 @@ func (s *toolingSuite) SetUpTest(c *C) {
 	s.BaseTest.AddCleanup(snap.MockSanitizePlugsSlots(func(snapInfo *snap.Info) {}))
 
 	s.tsto = tooling.MockToolingStore(s)
+	s.storeActionsBunchSizes = nil
+	s.storeActions = nil
+	s.curSnaps = nil
 
 	s.SeedSnaps = &seedtest.SeedSnaps{}
 	s.SetupAssertSigning("canonical")
-	s.Brands.Register("my-brand", brandPrivKey, map[string]interface{}{
+	s.Brands.Register("my-brand", brandPrivKey, map[string]any{
 		"verification": "verified",
 	})
 	assertstest.AddMany(s.StoreSigning, s.Brands.AccountsAndKeys("my-brand")...)
 
-	otherAcct := assertstest.NewAccount(s.StoreSigning, "other", map[string]interface{}{
+	otherAcct := assertstest.NewAccount(s.StoreSigning, "other", map[string]any{
 		"account-id": "other",
 	}, "")
 	s.StoreSigning.Add(otherAcct)
@@ -384,15 +389,184 @@ func (s *toolingSuite) TestDownloadSnap(c *C) {
 	opts := tooling.DownloadSnapOptions{
 		TargetDir: dlDir,
 	}
-	dlSnap, err := s.tsto.DownloadSnap("core", opts)
+	dlSnap, err := s.tsto.DownloadSnap("core", nil, opts)
 	c.Assert(err, IsNil)
 	c.Check(dlSnap.Path, Matches, filepath.Join(dlDir, `core_\d+.snap`))
 	c.Check(dlSnap.Info.SnapName(), Equals, "core")
 	c.Check(dlSnap.RedirectChannel, Equals, "")
+	c.Check(len(dlSnap.Components), Equals, 0)
+
+	c.Assert(s.storeActions, HasLen, 1)
+	// make sure that we do not provide a default channel here
+	c.Check(s.storeActions[0].Channel, Equals, "")
 
 	c.Check(logbuf.String(), Matches, `.* DEBUG: Going to download snap "core" `+opts.String()+".\n")
 }
 
+func (s *toolingSuite) TestDownloadSnapWithComps(c *C) {
+	dlDir := c.MkDir()
+	opts := tooling.DownloadSnapOptions{
+		TargetDir: dlDir,
+	}
+	s.testDownloadSnapWithComps(c, opts)
+}
+
+func (s *toolingSuite) TestDownloadOnlyComps(c *C) {
+	dlDir := c.MkDir()
+	opts := tooling.DownloadSnapOptions{
+		TargetDir:      dlDir,
+		OnlyComponents: true,
+	}
+	s.testDownloadSnapWithComps(c, opts)
+}
+
+func (s *toolingSuite) TestDownloadSnapWithCompsAndBasename(c *C) {
+	dlDir := c.MkDir()
+	opts := tooling.DownloadSnapOptions{
+		TargetDir: dlDir,
+		Basename:  "mybasename",
+	}
+	s.testDownloadSnapWithComps(c, opts)
+}
+
+func (s *toolingSuite) testDownloadSnapWithComps(c *C, opts tooling.DownloadSnapOptions) {
+	dlDir := opts.TargetDir
+	comRevs := map[string]snap.Revision{
+		"comp1": snap.R(22),
+		"comp2": snap.R(33),
+	}
+	s.SeedSnaps.MakeAssertedSnapWithComps(c, seedtest.SampleSnapYaml["required20"], nil,
+		snap.R(21), comRevs, "other", s.StoreSigning.Database)
+
+	// env shenanigans
+	runtime.LockOSThread()
+	defer runtime.UnlockOSThread()
+
+	debug, hadDebug := os.LookupEnv("SNAPD_DEBUG")
+	os.Setenv("SNAPD_DEBUG", "1")
+	if hadDebug {
+		defer os.Setenv("SNAPD_DEBUG", debug)
+	} else {
+		defer os.Unsetenv("SNAPD_DEBUG")
+	}
+	logbuf, restore := logger.MockLogger()
+	defer restore()
+
+	dlSnap, err := s.tsto.DownloadSnap("required20", []string{"comp1", "comp2"}, opts)
+	c.Assert(err, IsNil)
+	var snapPath, comp1Path, comp2Path string
+	if opts.Basename != "" {
+		if opts.OnlyComponents {
+			// The snap is not downloaded in this case
+			snapPath = ""
+		} else {
+			snapPath = filepath.Join(dlDir, opts.Basename+".snap")
+		}
+		comp1Path = filepath.Join(dlDir, opts.Basename+`\+comp1.comp`)
+		comp2Path = filepath.Join(dlDir, opts.Basename+`\+comp2.comp`)
+	} else {
+		if opts.OnlyComponents {
+			// The snap is not downloaded in this case
+			snapPath = ""
+		} else {
+			snapPath = filepath.Join(dlDir, "required20_21.snap")
+		}
+		comp1Path = filepath.Join(dlDir, `required20\+comp1_22.comp`)
+		comp2Path = filepath.Join(dlDir, `required20\+comp2_33.comp`)
+	}
+	c.Check(dlSnap.Path, Equals, snapPath)
+	c.Check(dlSnap.Info.SnapName(), Equals, "required20")
+	c.Check(dlSnap.RedirectChannel, Equals, "")
+	dlComps := dlSnap.Components
+	c.Check(len(dlComps), Equals, 2)
+	c.Check(dlComps[0].Path, Matches, comp1Path)
+	c.Check(dlComps[1].Path, Matches, comp2Path)
+	cref1 := naming.NewComponentRef("required20", "comp1")
+	c.Check(dlComps[0].Info, DeepEquals, &snap.ComponentInfo{
+		Component:         cref1,
+		Type:              snap.StandardComponent,
+		ComponentSideInfo: *snap.NewComponentSideInfo(cref1, snap.R(22)),
+	})
+	cref2 := naming.NewComponentRef("required20", "comp2")
+	c.Check(dlComps[1].Info, DeepEquals, &snap.ComponentInfo{
+		Component:         cref2,
+		Type:              snap.StandardComponent,
+		ComponentSideInfo: *snap.NewComponentSideInfo(cref2, snap.R(33)),
+	})
+
+	c.Check(logbuf.String(), Matches, `.* DEBUG: Going to download snap "required20" and component\(s\) "comp1", "comp2" `+opts.String()+".\n"+
+		"")
+}
+
+func (s *toolingSuite) TestDownloadManySnapWithComps(c *C) {
+	comRevs := map[string]snap.Revision{
+		"comp1": snap.R(22),
+		"comp2": snap.R(33),
+	}
+	s.SeedSnaps.MakeAssertedSnapWithComps(c, seedtest.SampleSnapYaml["required20"], nil,
+		snap.R(21), comRevs, "other", s.StoreSigning.Database)
+	s.setupSnaps(c, map[string]string{"core": "canonical"}, "")
+
+	// env shenanigans
+	runtime.LockOSThread()
+	defer runtime.UnlockOSThread()
+
+	snapsToDownld := []tooling.SnapToDownload{
+		{Snap: naming.Snap("core")},
+		{Snap: naming.Snap("required20"), CompsToDownload: []string{"comp1", "comp2"}},
+	}
+	dlDir := c.MkDir()
+	var numCore, numReq int
+	bdf := func(si *snap.Info, cinfos map[string]*snap.ComponentInfo) (targetPath string, compPaths map[string]string, err error) {
+		compPaths = make(map[string]string, len(cinfos))
+		switch si.SnapName() {
+		case "core":
+			c.Check(len(cinfos), Equals, 0)
+			numCore++
+		case "required20":
+			cref1 := naming.NewComponentRef(si.SnapName(), "comp1")
+			cref2 := naming.NewComponentRef(si.SnapName(), "comp2")
+			c.Check(cinfos, DeepEquals, map[string]*snap.ComponentInfo{
+				"comp1": {
+					Component:           cref1,
+					Type:                snap.StandardComponent,
+					ComponentProvenance: "global-upload",
+					ComponentSideInfo:   *snap.NewComponentSideInfo(cref1, snap.R(22)),
+				},
+				"comp2": {
+					Component:           cref2,
+					Type:                snap.StandardComponent,
+					ComponentProvenance: "global-upload",
+					ComponentSideInfo:   *snap.NewComponentSideInfo(cref2, snap.R(33)),
+				},
+			})
+			numReq++
+			compPaths[cref1.ComponentName] = filepath.Join(dlDir, fmt.Sprintf("%s.comp", cref1.String()))
+			compPaths[cref2.ComponentName] = filepath.Join(dlDir, fmt.Sprintf("%s.comp", cref2.String()))
+		default:
+			c.Error("unexpected snap", si.SnapName())
+		}
+		return filepath.Join(dlDir, si.SnapName()), compPaths, nil
+	}
+	topts := tooling.DownloadManyOptions{
+		BeforeDownloadFunc: bdf,
+		EnforceValidation:  false,
+	}
+	dss, err := s.tsto.DownloadMany(snapsToDownld, nil, topts)
+	c.Assert(err, IsNil)
+	c.Check(len(dss), Equals, 2)
+	c.Check(dss["core"].Info.SnapName(), Equals, "core")
+	c.Check(dss["required20"].Info.SnapName(), Equals, "required20")
+	c.Check(len(dss["core"].Components), Equals, 0)
+	comps := dss["required20"].Components
+	c.Check(len(comps), Equals, 2)
+	c.Check(comps[0].Info.Component, Equals, naming.NewComponentRef("required20", "comp1"))
+	c.Check(comps[1].Info.Component, Equals, naming.NewComponentRef("required20", "comp2"))
+
+	c.Check(numCore, Equals, 1)
+	c.Check(numReq, Equals, 1)
+}
+
 func (s *toolingSuite) TestSetAssertionMaxFormats(c *C) {
 	c.Check(s.tsto.AssertionMaxFormats(), IsNil)
 
@@ -440,12 +614,20 @@ func (s *toolingSuite) SnapAction(_ context.Context, curSnaps []*store.CurrentSn
 			redirectChannel = channel
 		}
 		info1.Channel = channel
+		comps := s.AssertedSnapComponents(a.InstanceName)
 		sars = append(sars, store.SnapActionResult{
 			Info:            &info1,
 			RedirectChannel: redirectChannel,
+			Resources:       comps,
 		})
 	}
 
+	// TODO: we can remove this source once we move to go >=1.20
+	r := rand.New(rand.NewSource(time.Now().UnixNano()))
+
+	// caller of SnapAction shouldn't depend on the order of the results
+	r.Shuffle(len(sars), func(i, j int) { sars[i], sars[j] = sars[j], sars[i] })
+
 	return sars, nil, nil
 }
 
@@ -505,15 +687,15 @@ func (s *toolingSuite) TestSimpleCreds(c *C) {
 }
 
 func (s *toolingSuite) setupSequenceFormingAssertion(c *C) {
-	vs, err := s.StoreSigning.Sign(asserts.ValidationSetType, map[string]interface{}{
+	vs, err := s.StoreSigning.Sign(asserts.ValidationSetType, map[string]any{
 		"type":         "validation-set",
 		"authority-id": "canonical",
 		"series":       "16",
 		"account-id":   "canonical",
 		"name":         "base-set",
 		"sequence":     "1",
-		"snaps": []interface{}{
-			map[string]interface{}{
+		"snaps": []any{
+			map[string]any{
 				"name":     "pc",
 				"id":       "idididididididididididididididid",
 				"presence": "required",
